/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

// sample connect/execute/disconnect to DL
/*
 * Filename       : TExecuteDL.cpp
 * Description    : Connect to DL Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DL Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobDL.hpp>
#include <DKPidXDODL.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;

    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TExecuteDL <libsrv> <userid> <pw>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKDatastoreDL dsDL; // must be outside try/catch block

    try {
    dkResultSetCursor* pCur = 0;
    cout << "Datastore DL created" << endl;
    cout << "connecting to datastore" << endl;
    dsDL.connect(libsrv,userid,pw);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    DKString cmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,";
    cmd += "MAX_RESULTS=5,";
    cmd += "COND=(DLSEARCH_DocType <> NULL));";
    cmd += "OPTION=(CONTENT=YES;";
    cmd += "TYPE_QUERY=STATIC;TYPE_FILTER=FOLDERDOC)";
    cout << "query string " << cmd << endl;
    cout << "executing query" << endl;
    pCur = dsDL.execute(cmd);
    cout << "query executed" << endl;

    DKDDO *p = 0;
    dkDataObjectBase *pDOBase = 0;
    DKDDO *pDDO = 0;
    dkXDO *pXDO = 0;
    DKAny a;
    ushort j = 0;
    ushort k = 0;
    ushort val = 0;
    ushort cnt = 1;
    DKString strData = "";
    DKString strDataName = "";
    dkCollection* pdCol = 0;
    dkIterator*  pdIter = 0;
    ushort numDataItems = 0;
	DKPidXDODL *pidXDO = 0;
	DKPid *pid = 0;
    DKString strPid;
	long pidIdCnt = 0;
	long pidIndex = 0;
    short typeProp = 0;
    DKTimestamp ts(1997, DKDate::May, 5, 10, 10, 10, 10);

    while (pCur->isValid())
    {
     p = pCur->fetchNext();
     if (p != 0)
     {
      cout << "==========> " << "Item " << cnt << " <=========" << endl;
      numDataItems = p->dataCount();
	  pid = (DKPid*)p->getPidObject();
	  strPid = pid->pidString();
	  cout << "pid string " << strPid << endl;
	  cout << "pid id string " << pid->getId() << endl;
	  strPid = pid->getIdString();
	  cout << "pid idString " << strPid << endl;
      pidIdCnt = pid->getIdStringCount();
	  cout << "pid idString cnt " << pidIdCnt << endl;
	  strPid = pid->getPrimaryId();
	  cout << "pid primary id " << strPid << endl;
	  pidIndex = 0;
	  strPid = pid->getIdString(pidIndex);
	  cout << "pid item id " << strPid << endl;
      k = p->propertyId(DK_CM_PROPERTY_ITEM_TYPE);
      if (k > 0)
      {
       a = p->getProperty(k);
       val = a;
       cout << "******************************" << endl;
       switch (val)
       {
        case DK_CM_DOCUMENT :
        {
  	     cout << "Item is document " << endl;
         break;
        }
	    case DK_CM_FOLDER :
        {
	     cout << "Item is folder " << endl;
         break;
        }
       }
       cout << "******************************" << endl;
      }
      k = p->propertyId(DK_CM_PROPERTY_CHECKOUT);
      if (k > 0)
      {
       a = p->getProperty(k);
       strData = a;
       cout << "checkout prop " << strData << endl;
      }
      k = p->propertyId(DK_CM_PROPERTY_CREATED_TIMESTAMP);
      if (k > 0)
      {
       a = p->getProperty(k);
       ts = a;
       strData = ts.asString(NULL);
       cout << "created timestamp prop " << strData << endl;
      }
      k = p->propertyId(DK_CM_PROPERTY_MODIFIED_TIMESTAMP);
      if (k > 0)
      {
       a = p->getProperty(k);
       ts = a;
       strData = ts.asString(NULL);
       cout << "modified timestamp prop " << strData << endl;
      }

      cout << "Number of Data Items " << numDataItems << endl;
      for (j = 1; j <= numDataItems; j++)
      {
       a = p->getData(j);
       strDataName = p->getDataName(j);
       k = p->dataPropertyId(j,DK_CM_PROPERTY_TYPE);
       if (k > 0)
       {
      	a = p->getDataProperty(j,k);
        typeProp = a;  // (ie DK_CM_DATAITEM_TYPE_STRING)
	//	cout << "type property " << typeProp << endl;
       }
       switch (a.typeCode())
       {
        case DKAny::tc_collection :
        {
         pdCol = a;
         cout << strDataName << " collection name : " << strDataName << endl;
         cout << "--------------------------------------" << endl;
         pdIter = pdCol->createIterator();
         ushort b = 0;
         while (pdIter->more() == TRUE)
         {
	      b++;
          cout << "  -------------------------------------------------" << endl;
	      a = *(pdIter->next());
	      pDOBase = a;
	      if (pDOBase->protocol() == DK_CM_PDDO)
          {
	       pDDO = (DKDDO*)pDOBase;
	       cout << "  DKDDO object " << b << " in " << strDataName << " collection " << endl;
	       k = pDDO->propertyId(DK_CM_PROPERTY_ITEM_TYPE);
	       if (k > 0)
           {
	        a = pDDO->getProperty(k);
	        val = a;
	        cout << "  ******************************" << endl;
	        switch (val)
            {
	         case DK_CM_DOCUMENT :
             {
              cout << "  Item is document " << endl;
	          break;
             }
	         case DK_CM_FOLDER :
             {
	          cout << "  Item is folder " << endl;
	          break;
             }
            }
	        cout << "  ******************************" << endl;
           }
          }
	      else if (pDOBase->protocol() == DK_CM_XDO)
          {
	       pXDO = (dkXDO*)pDOBase;
	       cout << "  dkXDO object " << b << " in " << strDataName << " collection " << endl;
	       pidXDO = (DKPidXDODL*)pXDO->getPid();
  	       strPid = pidXDO->pidString();
	       cout << "XDO pid string " << strPid << endl;
	       strPid = pidXDO->getId();
	       cout << "XDO pid id string " << strPid << endl;
	       strPid = pidXDO->getIdString();
	       cout << "XDO pid idString " << strPid << endl;
           pidIdCnt = pidXDO->getIdStringCount();
 	       cout << "XDO pid idString cnt " << pidIdCnt << endl;
	       strPid = pidXDO->getPrimaryId();
	       cout << "XDO pid primary id " << strPid << endl;
	       pidIndex = 0;
	       strPid = pidXDO->getIdString(pidIndex);
	       cout << "XDO pid item id " << strPid << endl;
	       pidIndex = 1;
	       strPid = pidXDO->getIdString(pidIndex);
	       cout << "XDO pid part no " << strPid << endl;
	       pidIndex = 2;
	       strPid = pidXDO->getIdString(pidIndex);
	       cout << "XDO pid rep type " << strPid << endl;
          }
         }

         if (pdIter != 0)
         {
          delete pdIter;
         }
         if (b == 0)
         {
	      cout << strDataName << " collection has no elements " << endl;
         }
         cout << "  -------------------------------------------------" << endl;
         break;
        }
        case DKAny::tc_null :
        {
         cout << "attribute name : " << strDataName << " value : null " << endl;
         break;
        }
        default:
        {
         cout << "attribute name : " << strDataName << " value :" << a << endl;
         break;
        }
       }
      }
      cnt++;
      delete p;
     }
    }
     cout << "Total Item count is " << cnt-1 << endl;
     if (pCur != 0)
       delete pCur;
     dsDL.disconnect();
     cout << "datastore disconnected" << endl;
    }
    catch(DKException &exc)
    {

      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;

}

