/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

When Updating Reference Attributes, use it as an Attribute Group.  Updating Reference 
Attribute Definitions work the same as Attribute Groups, but no attributes may be added 
or removed.  Updating it is simply for the other options, such as name, description,
and delete rule.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SReferenceAttrDefRetrievalICM.hpp"
#include "SReferenceAttrDefUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: SReferenceAttrDefUpdateICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, & Reference Attribute Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Reference Attribute Definitions in Memory
 *                    Saving / Updating Reference Attribute Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SReferenceAttrDefUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Reference Attributes used here must already exist.  If not, please
 *                    run the SReferenceAttrDefCreationICM Sample first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SReferenceAttrDefRetrievalICM.hpp/cpp
 *                    SReferenceAttrDefUpdateICM_launch.cpp
 *                    SReferenceAttrDefUpdateICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SReferenceAttrDefUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                        << endl;
            cout << "  SReferenceAttrDefUpdateICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."         << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SReferenceAttrDefUpdateICM"     << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Retrieve a Reference Attribute (Group)
            //--------------------------------------------------------------
            // Treat the Reference Attribute (Group) as an actual
            // Attribute Group, since it was created as such.
            //
            // Retrieving the "S_myRef" Reference Attribute (Group) Definition.
            cout << "Retrieving Reference Atttribute 'S_myRef'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_myRef"); // Retrieve the definition.

            cout << "Retrieved Reference Attribute 'S_myRef'." << endl;

            //--------------------------------------------------------------
            // Modify an Reference Attribute in Memory
            //--------------------------------------------------------------
            // Updating Reference Attribute Definitions work the same as 
            // Attribute Groups, but no attributes may be added or removed.
            // Updating it is simply for the other options, such as name, 
            // description, and delete rule.
            //
            // This example will modify the description of the S_myRef Reference
            // Attribute.
            cout << "Modifying Reference Attribute Definition 'S_myRef' in Memory..." << endl;
            
                attrGroupDef->setDescription("My Modified Reference Attribute");  // Change the description
                
            cout << "Modified Reference Attribute Definition 'S_myRef' in Memory." << endl;
            
            //--------------------------------------------------------------
            // Save / Update Reference Attribute Definition
            //--------------------------------------------------------------
            // The Attribute Group Definition's update() method can be called
            // as well as the Datastore Definition's update(attrGroupDef) method.
            //
            // This example will use the Attribute Group's update() method.
            cout << "Saving / Updating Reference Attribute Definition 'S_myRef' in Datastore..." << endl;

                attrGroupDef->update();  // Save / Update Changes in Datastore Definition

            cout << "Updated Reference Attribute Definition 'S_myRef' in Datastore." << endl;

            //-------------------------------------------------------------
            // Print Updated Reference Attribute (Group) Definition
            //-------------------------------------------------------------
            cout << "Printing the Updated Reference Attribute Definition of 'S_myRef'..." << endl;

                SReferenceAttrDefRetrievalICM::printReferenceAttrDef(attrGroupDef); // Use wrapper funciton from the Retrieval Sample.
                
                delete(attrGroupDef);     // Free Memory

            cout << "Printed the Updated Reference Attribute Definition of 'S_myRef'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
