/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.beans.*;

/**
 * This sample demonstrates using the CMBConnection and CMBSchemaManagement
 * beans, showing how events on the CMBSchemaManagement beans can be delegated
 * to the event listeners on the CMBConnection bean.
 */
public class DemoConnectWithSchema {
    UtilPersistent per = new UtilPersistent();
    UtilSimpleUI ui = new UtilSimpleUI();
    DemoConnect connDemo = new DemoConnect();
    CMBSchemaManagement schemaBean = null;
    CMBTraceLog traceBean = null;

    DemoConnectWithSchema() {
        connDemo.connBean.setTraceEnabled(true);
        schemaBean = connDemo.connBean.getSchemaManagement();
        // get schema from connection
        schemaBean.setTraceEnabled(true);

        try {
            System.out.println("Creating CMBTraceLog bean using " + "ser/class/jar specified in the classpath.");
            traceBean = (com.ibm.mm.beans.util.CMBTraceLog) Beans.instantiate(null, "com.ibm.mm.beans.util.CMBTraceLog");
            traceBean.setProgressDisplayEnabled(true);
            traceBean.setDebugDisplayEnabled(true);
            traceBean.setErrorDisplayEnabled(true);
            traceBean.setDisplayMode(CMBUtilConstant.DISPLAY_MESSAGE_IN_WINDOW);

            connDemo.connBean.addCMBTraceListener(traceBean);
            // no need to add trace listener to
            // schema bean, connection bean
            // deledates schema's trace (& exc)
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void runSchema() // copied fro DemoSchema.java
    {
        int count;
        try {
            // get server names
            System.out.println("Getting server names...");
            String svNames[] = schemaBean.getServerNames();
            count = svNames.length;
            System.out.println("Done, server count is: " + count);
            for (int i = 0; i < count; ++i)
                System.out.println("Server[" + i + "] = " + svNames[i]);
            System.out.println("");

            // get search template names
            System.out.println("Getting search template names...");
            String stNames[] = schemaBean.getSearchTemplateNames();
            count = stNames.length;
            System.out.println("Done, search template count is: " + count);
            for (int i = 0; i < count; ++i)
                System.out.println("Template[" + i + "] = " + stNames[i]);
            System.out.println("");

            // get entity names
            System.out.println("Getting entity names...");
            String entNames[] = schemaBean.getEntityNames();
            count = entNames.length;
            System.out.println("Done, entity count is: " + count);
            String attrNames[] = null;
            int attrCount = 0;
            for (int i = 0; i < count; ++i) {
                // for each entity, get attribute names
                System.out.println("Entity[" + i + "] = " + entNames[i]);
                attrNames = schemaBean.getEntityAttrNames(entNames[i]);
                attrCount = attrNames.length;
                System.out.println("Entity[" + i + "] = " + entNames[i] + ", has " + attrCount + " attributes");
                for (int j = 0; j < attrCount; ++j) {
                    System.out.println("  Attribute[" + j + "] = " + attrNames[j]);
                }
            }
            System.out.println("");

        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    public static void main(String argv[]) {
        String demoName = "IBM CMB Demonstration program: DemoConnectWithSchema";
        String copyright = "C) Copyright IBM Corp. 1994,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(demoName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        UtilSimpleUI ui = new UtilSimpleUI();
        DemoConnectWithSchema demo = new DemoConnectWithSchema();
        boolean loopControl = true;

        while (loopControl) {
            System.out.println("");
            System.out.println("Selections:");
            System.out.println("0. Exit");
            System.out.println("1. Try connect");
            System.out.println("2. Try schema functions");
            System.out.println("3. Try disconnect");
            switch (ui.getInteger("Enter your selection:")) {
                case 0 :
                    loopControl = false;
                    break;
                case 1 :
                    demo.connDemo.connectUsingEvent();
                    break;
                case 2 :
                    demo.runSchema();
                    break;
                case 3 :
                    demo.connDemo.disconnectUsingEvent();
                    break;
            }
        }

        //Calling disconnect from connDemo
        demo.connDemo.disconnect();

        //done, exiting....
        System.out.println("Bye.");
        System.exit(0);
    }
}