/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TDatastorePoolDB2.java
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.cs.*;
import java.io.*;
import java.lang.reflect.*;

// TDatastorePoolDB2 class
public class TDatastorePoolDB2 implements DKConstantDB2 {

  // Main method
  public static void main(String argv[]){
    
      //*** for input parameters
      String database = new String(""), // database - SAMPLE
             userid = new String(""),   // userid 
  	     pw = new String(""),       // pw
             schema = new String("");   // schema   

      String fullClassName = "com.ibm.mm.sdk.server.DKDatastoreDB2";

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 

      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TDatastorePoolDB2 " +
        "<database> <userid> <pw> [<schema-name> - optional]" );
       return;
      }
      else if (argv.length == 4)
      {
       schema = argv[3];
      }
      database = argv[0];
      userid = argv[1];
      pw = argv[2];


    try {
    DKDatastorePool dsrDB2 = new DKDatastorePool(fullClassName);
    dsrDB2.setDatastoreName(database);
    dsrDB2.setMaxPoolSize(10);
    dsrDB2.setMinPoolSize(3);

    dsrDB2.initConnections(userid, pw, 2);
    dsrDB2.setTimeOut(1); 
    System.out.println(" getMaxPoolSize =  " + dsrDB2.getMaxPoolSize());
    System.out.println(" getMinPoolSize =  " + dsrDB2.getMinPoolSize());
    System.out.println(" getTimeOut=  " + dsrDB2.getTimeOut());

    dkDatastore dsDB21, dsDB22, dsDB23, dsDB24;
    dsDB21 = dsrDB2.getConnection(userid, pw);             // new ds1
    System.out.println(" getConnection, dsDB21(ds1) =  " + dsDB21);
    dsDB22 = dsrDB2.getConnection(userid, pw);             // new ds2
    System.out.println(" getConnection, dsDB22(ds2) =  " + dsDB22);

    DKDatastoreSession dsIdDB21, dsIdDB22, dsIdDB23, dsIdDB24;
    dsIdDB21 = dsrDB2.getConnection(userid, pw, 0);        //new dsid1
    System.out.println(" getConnection, dsIdDB21(dsid1)=" +
     dsIdDB21.getDatastore() + ", sessionId=" + dsIdDB21.getId());
    dsIdDB22 = dsrDB2.getConnection(userid, pw, 0);        //new dsid2
    System.out.println(" getConnection, dsIdDB22(dsid2)=" +
     dsIdDB22.getDatastore() + ", sessionId=" + dsIdDB22.getId()); 

    dsrDB2.returnConnection(dsIdDB22);                     //dsid2 in _freeIdPool
    System.out.println(" returnConnection, dsIdDB22(dsid2) =  " + 
      dsIdDB22.getDatastore() + ", sessionId=" + dsIdDB22.getId());
    dsrDB2.returnConnection(dsDB22);                       //ds2 in _freePool
    System.out.println(" returnConnection, dsDB22(ds2) =  " + dsDB22); 

    dsIdDB23 = dsrDB2.getConnection(userid, pw, 1); //ds2 in _usedPool
    System.out.println(" getConnection, dsIdDB23(ds2)=" +
     dsIdDB23.getDatastore() + ", sessionId=" + dsIdDB23.getId());
    dsIdDB24 = dsrDB2.getConnection(userid, pw, 5); //new dsid3  
    System.out.println(" getConnection, dsIdDB24(dsid3)=" +
     dsIdDB24.getDatastore() + ", sessionId=" + dsIdDB24.getId());


    dsIdDB24 = dsrDB2.getConnection(userid, pw, 6); // new dsid4
    System.out.println(" getConnection, dsIdDB24(dsid4)=" +
     dsIdDB24.getDatastore() + ", sessionId=" + dsIdDB24.getId());

    dsDB21 = dsrDB2.getConnection(userid, pw);  // new ds3
    System.out.println(" getConnection, dsDB21(ds3) =  " + dsDB21);
    dsDB22 = dsrDB2.getConnection(userid, pw);  // new ds4
    System.out.println(" getConnection, dsDB22(ds4) =  " + dsDB22); 

    dsDB23 = dsrDB2.getConnection(userid, pw);  // new ds5
    System.out.println(" getConnection, dsDB23(ds5) =  " + dsDB23);
    dsDB24 = dsrDB2.getConnection(userid, pw);  // new ds6
    System.out.println(" getConnection, dsDB24(ds6) =  " + dsDB24); 


    dsIdDB23 = dsrDB2.getConnection(userid, pw, 9); // dsid2 
    System.out.println(" getConnection, dsIdDB23(dsid2)=" +
     dsIdDB23.getDatastore() + ", sessionId=" + dsIdDB23.getId());


    //**** use dsDB22 to do query
/**/
    System.out.println("||**** start: use dsDB22 to do query ****||");       
       dkQuery pQry = null;
       String cmd = "";
       DKResults pResults = null;
       DKNVPair parms1[] = null;
       DKNVPair parms[] = new DKNVPair[2];
       String strMax = "5";
       parms[0] = new DKNVPair(DK_CM_PARM_MAX_RESULTS,strMax);
       parms[1] = new DKNVPair(DK_CM_PARM_END,null);
       System.out.println("connecting to datastore");
       if (schema.equals("")) 
        cmd = "SELECT * FROM EMPLOYEE";
       else
        cmd = "SELECT * FROM " + schema + ".EMPLOYEE"; 
       System.out.println("query string " + cmd);
       System.out.println("create query");
       pQry = dsDB22.createQuery(cmd,DK_CM_SQL_QL_TYPE,parms1);
       System.out.println("executing query");
       pQry.execute(parms);
       System.out.println("query executed");
       System.out.println("number of results " + pQry.numberOfResults());
       System.out.println("process query results");
       pResults = (DKResults)pQry.result();
       processResults((dkCollection)pResults);
       System.out.println("query results processed");
       dsDB22.commit();
    System.out.println("||**** end: use dsDB22 to do query ****||");
/**/
    //****end of use dsDB22 to do query 

    dsrDB2.returnConnection(dsDB22);
    System.out.println(" returnConnection, dsDB22(ds4) =  " + dsDB22);
    dsDB24 = dsrDB2.getConnection(userid, pw);
    System.out.print(" getConnection, dsDB24 =  " + dsDB24); 
    System.out.println("    dsDB24 == dsDB22 =  " + (dsDB24 == dsDB22)); //true

    //**** use dsDB24 to retrieve XDO - need to change the itemId and partId
/**  
    System.out.println("||**** start: use dsDB24 to retrieve XDO ****||");
      
      String table;                                   //from SAMPLE database of DB2
      String column = "RESUME";                       //clob column 
      String predicate = "EMPNO = '000130' and RESUME_FORMAT = 'ascii'" ; //primary key of exist clob
      if (schema.equals("")) 
        table = "EMP_RESUME";
      else
        table = schema + ".EMP_RESUME"; 
      DKClobDB2 axdo = new DKClobDB2(dsDB24);
      DKPidXDODB2  apid = new DKPidXDODB2();
      apid.setTable(table);
      apid.setColumn(column);
      apid.setDataPredicate(predicate);   //exist clob
      axdo.setPidObject(apid);
      System.out.println("table= " + axdo.getTable());
      System.out.println("column=" + ((DKPidXDODB2)(axdo.getPidObject())).getColumn());
      System.out.println("dataPredicate=" + ((DKPidXDODB2)(axdo.getPidObject())).getDataPredicate());
      //### retrieve the exist clob - 000130 ascii
      System.out.println("about to do retrieve()...");
      axdo.retrieve();
      System.out.println("retrieve successfully...");    
      System.out.println("about to do open()");
      axdo.setInstanceOpenHandler("notepad ", true);   //use notepad in NT
      axdo.open();     //iexplore requires the file extension
      dsDB24.commit();
    System.out.println("||**** end: use dsDB24 to retrieve XDO ****||");
**/
    //****end of use dsDB24 to retrieve XDO
       
    dsrDB2.returnConnection(dsDB21);
    System.out.println(" returnConnection, dsDB21(ds3) =  " + dsDB21);
    dsrDB2.returnConnection(dsDB23);
    System.out.println(" returnConnection, dsDB23(ds5) =  " + dsDB23);
    dsrDB2.returnConnection(dsIdDB24); //dsid4 in _freeIdPool
    System.out.println(" returnConnection, dsIdDB24(dsid4) =  " + 
      dsIdDB24.getDatastore() + ", sessionId=" + dsIdDB24.getId());
     

    System.out.println("**** start sleep for 120050 mini sec*******");
    Thread.sleep(120050);  //the two in freePool will be disconnected
    System.out.println("**** end sleep for 120050 mini sec*******");

    dsDB22 = dsrDB2.getConnection(userid, pw);     //new ds7
    System.out.println(" getConnection, dsDB22(ds7) =  " + dsDB22); 
    System.out.println(" call destroy()..." );
    dsrDB2.destroy();  //take a while for all 4 in usedPool to be disconnected

    }
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }

  public static void processResults(dkCollection results) 
                throws DKException, Exception
  {
    Short    sVal;
    Integer  iVal;
    short    itemType = 0;
    DKDDO    item = null;
    short    i = 0;
    String   dataName;
    DKParts  parts = null;
    DKFolder folder = null;
    System.out.println( "number of items " + results.cardinality());
 
    dkIterator iter = results.createIterator();
 
    while(iter.more()) {
       item = (DKDDO) iter.next();
       if (item != null) {
	   //   	  item.retrieve(); //Only valid if Primary Key exists for Table
   	  sVal = (Short)item.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
   	  itemType = sVal.shortValue();
   	  switch(itemType) {
   	    case DK_CM_DOCUMENT:
	      {
   	      System.out.println(" Item is a document, number of data item : " + item.dataCount());
   	      for (i = 1; i <= item.dataCount(); i++) {
   		   dataName = item.getDataName(i);
   		   System.out.println("    data-item[" + i + "] = " + dataName + ", value " + item.getData(i));
   		   }
   	      }
   	      break;
   	    default:
   	      System.out.println("  Item is not recognized ");
   	      break;
   	  } 
       }
    } 
  }
}

