/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample of xdo BLOB for DB2

/*
 * Filename       : TxdoCRUDDB2.java
 * Description    : test add, update, delete, retrieve of BLOB for DB2 - thru memory.
 *                  (use the DB2 SAMPLE database, table EMP_PHOTO) 
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.internal.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TxdoCRUDDB2 class
public class TxdoCRUDDB2 implements DKConstantDB2
{

  // Main method
  public static void main(String argv[])
  {
          
      //*** for input parameters
      String database = new String(""),   // database
             userid = new String(""),     //userid 
  	     pw = new String(""); // pw

      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
     
      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TxdoCRUDDB2 " +
        "<database> <userid> <pw>" );
       return;
      }
      else
      {
       database = argv[0];
       userid = argv[1];
       pw = argv[2];
      } 

      //### setup table, colume, predicate, picture files
      String fileName1 =  "e:\\cmbroot\\samples\\java\\dl\\images\\eagle.gif"; //for add
      String fileName2 =  "e:\\cmbroot\\samples\\java\\dl\\images\\vir2.gif";   //for update web.gif
      String table = "EMP_PHOTO";                           //from SAMPLE database of DB2
      String column = "PICTURE";                            //blob column 
      String predicate0 = "EMPNO = '000130' and PHOTO_FORMAT = 'gif'" ; //primary key of exist blob
      String predicate1 = "EMPNO = '000210' and PHOTO_FORMAT = 'gif'" ; //primary key for null blob

      DKDatastoreDB2 dsDB2 = null;
      try {
      // Constructing using default constuctor with init method
      dsDB2 = new DKDatastoreDB2();
      System.out.println("connecting to datastore"); 
      dsDB2.connect(database,userid,pw,"");
      System.out.println("datastore connected " + database + " userid - " + userid);

      //### setup XDO
      DKBlobDB2 axdo = new DKBlobDB2(dsDB2);
      DKPidXDODB2  apid = new DKPidXDODB2();
      apid.setTable(table);
      apid.setColumn(column);
      apid.setDataPredicate(predicate0);   //exist blob
      axdo.setPidObject(apid);
      System.out.println("table= " + axdo.getTable());
      System.out.println("column=" + ((DKPidXDODB2)(axdo.getPidObject())).getColumn());
      System.out.println("dataPredicate=" + ((DKPidXDODB2)(axdo.getPidObject())).getDataPredicate());

      //### retrieve the exist blob - 000130 gif
      System.out.println("about to do retrieve()...");
      axdo.retrieve();
      System.out.println("retrieve successfully...");    
      System.out.println("about to do open()");
      axdo.setInstanceOpenHandler("lviewpro", true);   //use lviewpro in NT
      axdo.open("000130.gif");     //iexplore requires the file extension  

      //### add from memory to the null blob - 000210 gif
      apid.setDataPredicate(predicate1);  //null blob
      axdo.setPidObject(apid);
      System.out.println("table= " + axdo.getTable());
      System.out.println("column=" + axdo.getColumn());
      System.out.println("dataPredicate=" + axdo.getDataPredicate());
      axdo.setContentFromClientFile(fileName1);
      System.out.println("before add length(in memory)= " + axdo.length());  //in memory
      System.out.println("before add size(in datastore)= " + axdo.getSize()); //in datastore
      System.out.println("about to do add()....");
      axdo.add();       
      System.out.println("add successfully.... ");

      //### retrieve to memeory and open - 000210 gif
      System.out.println("about to do retrieve() of added blob...");
      axdo.retrieve();
      System.out.println("retrieve successfully...");
      System.out.println("about to do open()");
      axdo.setInstanceOpenHandler("lviewpro ", true);   //use lviewpro in NT
      axdo.open("000210.gif");     //iexplore requires the file extension 

      //### update the added blob - 00210 gif
      //== before update, the content length is the same
      System.out.println("before update, content length = " + axdo.getContent().length);
      axdo.setContentFromClientFile(fileName2); //set XDO content from a file
      System.out.println("about to do update()...");
      axdo.update();  
      System.out.println("After update, do retrieve and open() to verify it is updated.");
      axdo.retrieve();
      System.out.println("after update & retrieve, content length = " + axdo.length());
      axdo.open("000210.gif");     //iexplore requires the file extension 

      //### delete the added blob - 000210 gif 
      System.out.println("Delete the above added blob ...");
      axdo.del();
      System.out.println("delete successfully!");

      dsDB2.commit();
      dsDB2.disconnect();
      System.out.println("datastore disconnected"); 
      dsDB2.destroy();
    }
    catch (DKException exc)
    {
     try {
     dsDB2.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
     dsDB2.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}

