/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//*****************************************************************************
//
// Sample class to search Domino.Doc library for documents which
// satisfy search criteria.
//
//*****************************************************************************

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.util.Vector;



 public class TClientEvaluateDD implements DKConstantDD
 {
   //================================================================
   // Constructors
   //================================================================
   public void TClientEvaluateDD(){}

   //================================================================
   // main
   //================================================================
   public static void main(String argv[])
   {
     System.out.println(" ");
     System.out.println(Me + ": Starting");
     String str;

     TClientEvaluateDD thisTest = new TClientEvaluateDD();

     // Input parameters
     String libURL = new String("");   // library server
     String userid = new String("");
     String pw     = new String("");
     // Indicate this is a Console Application on Windows NT/95 for error
     // message display in dos console code page. If this was a Windows
     // GUI (Graphic User Interface) Application this would not be necessary
     // because DK_CM_SS_WINDOWS is the default.
     DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

     //--------------------------------------------------------------
     // Checking for input parameters
     //--------------------------------------------------------------

     if (argv.length < 3)
     {
       System.out.println(Me + " Usage: " );
       System.out.println("  java " + Me +
       " <libURL> <userid> <pw>" );
       return;
     }
     else
     {
       libURL = argv[0];
       userid = argv[1];
       pw = argv[2];
       System.out.println(Me + ": Parameter1=" + argv[0] );
       System.out.println(Me + ": Parameter2=" + argv[1] );
       System.out.println(Me + ": Parameter3=" + argv[2] );
     }

     DKDatastoreDD dsDD = null;
     try
     {
       dsDD = new DKDatastoreDD();

       //-----------------------------------------------------------------------------
       // get Connection
       //-----------------------------------------------------------------------------
       System.out.println("\n" + Me + ": Connecting to datastore");
       dsDD.connect(libURL,userid,pw,"");

       //-----------------------------------------------------------------------------
       // Evaluate a simple lotus query
       //-----------------------------------------------------------------------------
       String qString = qString2;
       System.out.println("\n\nEvaluate a simple lotus query: " + qString );

       String queryParms = "ENTITY=\"Diane Cabinet\" QUERY=" + qString;
       DKResults simpleResult = (DKResults) dsDD.evaluate(queryParms,  DK_CM_UNKNOWN_QL_TYPE, null);
       dkIterator itResults = simpleResult.createIterator();
       int i = 1;
       while( itResults.more() )
       {
         DKDDO aDDO = (DKDDO)itResults.next(); // get the next document out of the result list
         thisTest.printDDO( aDDO, i++ );
       }


       //-----------------------------------------------------------------------------
       // Evaluate a DKParametricQuery (which implements dkQuery interface)
       //-----------------------------------------------------------------------------

       System.out.println("\n\nEvaluate a DKParametricQuery (which implements dkQuery interface)");
       // build a parametric query
       DKNVPair[] params = null;
       String queryString = "ENTITY=\"Diane Cabinet\" QUERY=" + qString2;

       DKParametricQuery paramQuery = (DKParametricQuery) dsDD.createQuery( queryString
          , DK_CM_PARAMETRIC_QL_TYPE
          , params);

       // submit the query
       DKResults dkQResults = (DKResults) dsDD.evaluate( paramQuery );

       // display results
       itResults = dkQResults.createIterator();
       i = 1;
       while( itResults.more() )
       {
         DKDDO aDDO = (DKDDO)itResults.next();  // get the nex document out of the result list
         thisTest.printDDO( aDDO, i++ );
       }





       //-----------------------------------------------------------------------------
       // Evaluate a Compound query expression
       //-----------------------------------------------------------------------------
       System.out.println("\n\nEvaluate a Compound query expression");

       // build a compound query which contains just one parametric query
       DKCQExpr ccQueryExpr = thisTest.createQueryExpr();

       // submit the query
       DKResults exprResult = (DKResults) dsDD.evaluate( ccQueryExpr );

       // display results
       itResults = exprResult.createIterator();
       i = 1;
       while( itResults.more() )
       {
         DKDDO aDDO = (DKDDO)itResults.next();  // get the nex document out of the result list
         thisTest.printDDO( aDDO, i++ );
       }

       //-----------------------------------------------------------------------------
       // Evaluate a parametric query which was built from a query expression
       //-----------------------------------------------------------------------------
       System.out.println("\n\nEvaluate a parametric query built from a DKCQExpr");

       // build a parametric query using the expression from the previous evaluation
       paramQuery = (DKParametricQuery) dsDD.createQuery( ccQueryExpr );

       // submit the query
       exprResult = (DKResults) dsDD.evaluate( paramQuery );

       // display results
       itResults = exprResult.createIterator();
       i = 1;
       while( itResults.more() )
       {
         DKDDO aDDO = (DKDDO)itResults.next();  // get the nex document out of the result list
         thisTest.printDDO( aDDO, i++ );
       }


       //-----------------------------------------------------------------------------
       // disconect
       //-----------------------------------------------------------------------------
       System.out.println("\n");
       System.out.println(Me + ": Disconnecting from datastore");
       dsDD.disconnect();
       dsDD.destroy();
     }
     catch (DKException exc)
     {
       try {
       dsDD.destroy();
       }
       catch (Exception e)
       {
     	e.printStackTrace();
       } 
       System.out.println(Me + ": DKException message " + exc.getMessage());
       exc.printStackTrace();
     }
     catch(Exception exc)
     {
       try {
       dsDD.destroy();
       }
       catch (Exception e)
       {
     	e.printStackTrace();
       } 
       System.out.println(Me + ": Exception message " + exc.getMessage());
       exc.printStackTrace();
     }
     System.out.println(Me + ": Finished");
   }

  //=================================================================
  // Print DDO
  //=================================================================
  private void printDDO( DKDDO pDDO, int resultCounter )
    throws DKException, Exception
  {

    if (pDDO != null)
    {
       // pDDO.retrieve();  // used when CONTENT=NO
    }
    short pType = 0;
    System.out.println("=====> Item " + resultCounter + " <=====");
    int numDataItems = pDDO.dataCount();
    pType = pDDO.propertyId(DK_CM_PROPERTY_TYPE);
    if (pType > 0)
    {
       Short sVal = (Short)pDDO.getProperty(pType);
       short iType = sVal.shortValue();
       switch (iType)
       {
          case DK_DOCUMENT :
          {
             System.out.println("DDO represents a document ");
             break;
          }
          default:
          {
            System.out.println("Unexpected DDO type = " + iType + "(" + pDDO.getPidObject().getObjectType() + ")" );
          }
       }  // end switch...
    }  // end valid property

    System.out.println("Pid                  : " + pDDO.getPidObject().pidString());
    System.out.println("Number of Data Items : " + numDataItems);
    for (short j = 1; j <= numDataItems; j++)
    {
       Object attr = pDDO.getData(j);
       String strDataName = pDDO.getDataName(j);
       System.out.println(j + ": Attribute Name   : " + strDataName );
       if (attr instanceof String)
       {
          System.out.println("   String Value     : " + attr);
       }
       else
       {
          System.out.println("   Unknown Attribute type, Value " + attr);
       }
    }  // endfor
    return;
  }

   //================================================================
   // Build a ContentConnect sample Query Expression
   //================================================================
   private DKCQExpr createQueryExpr()
    throws DKException, Exception
   {
     DKCQExpr cCompoundQueryExpr = new DKCQExpr();
     DKQExpr  cQueryExpr = new DKQExpr();

     DKQTerm qTerm1 = new DKQTerm("Title", DKOpCode.LIKE, "*" );
     //DKQTerm qTerm1 = new DKQTerm("InvalidFieldName", DKOpCode.LIKE, "*");
     DKQTerm qTerm2 = new DKQTerm("DocFormat", DKOpCode.LIKE, "NOTES" );
     DKQTerm qTerm3 = new DKQTerm( (DKQTerm)null, DKOpCode.NOT, qTerm2 );

     DKQTerm qTerm4 = new DKQTerm( qTerm3, DKOpCode.AND, qTerm1 );

     DKQSubExpr cQuerySubExpr = new DKQSubExpr();

     cCompoundQueryExpr.setQLType( DK_CM_PARAMETRIC_QL_TYPE );
     cQueryExpr.setQLType( DK_CM_PARAMETRIC_QL_TYPE );

     cQuerySubExpr.setQueryTerm(qTerm4);
     cQueryExpr.addSubQuery( cQuerySubExpr );
     cCompoundQueryExpr.setQueryExpr( cQueryExpr );

     return cCompoundQueryExpr;
   }


   //================================================================
   // Data for this class
   //================================================================

   private static String Me = new String("TEvaluateDD");
   // qString1 produces nothing.
   private static String qString1 = new String("(NOT([\"\"DocFormat\"\"]CONTAINS\"\"NOTES\"\"))AND(NOT([DocType]CONTAINS\"\"Junk\"\"))");
   private static String qString2 = new String("((NOT([\"DocFormat\"]CONTAINS\"NOTES\"))  AND  ([\"Description\"]CONTAINS\"Request*\"))");
   //
   private static String cond1 = new String("(NOT([DocFormat]CONTAINS\"NOTES\"))");
   private static String cond2 = new String("([DocType] CONTAINS \"*\")");

 }

