/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TADLsmTM.java
 * Description    : test add, delete, list schema-mappings
 *                : This program will create a fed entity 'FedTMIndex' which mapped to
 *                  TMIndex on Mufasa.
 *
 */

import java.sql.*;
import java.lang.*;
import java.util.*;
import com.ibm.mm.sdk.common.dkSchemaMapping;
import com.ibm.mm.sdk.common.dkAttrMapping;
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;

class TADLsmTM implements DKConstantFed, DKConstantIP, DKConstantV4, DKConstantDL
{
    public static void main(String argv[]) 
    { 
        //-------------------------------------------------------------- 
        // Checking for input parameters
        //-------------------------------------------------------------- 
        if (argv.length < 4) {
           System.out.println("Usage Syntax: ");
           System.out.println("\t java TADLsmTM <feddb> <userid> <pw> <add/del/list>" );
           return;
        }
        
        String libSrv = argv[0];
        String userid = argv[1];
        String pw     = argv[2];
        String cmd    = argv[3];

        try {
            // create federated datastore
            System.out.println(" *** connecting to datastore : " + libSrv);
            dkDatastore ds = new DKDatastoreFed();
            ds.connect(libSrv, userid, pw, "");
            System.out.println(" *** datastore connected *** "); 

            // get the Def
            dkDatastoreDef  defFed = ds.datastoreDef();

            // associate data sources to this entity:
            // get the Fed DS Def and Admin
            DKDatastoreAdminFed adm = (DKDatastoreAdminFed) defFed.datastoreAdmin();

            // create a federated entity : FedTMINDEX
            DKEntityDefFed entityObj = new DKEntityDefFed(ds);        
               entityObj.setName("FedTMINDEX");
               entityObj.setDescription("FedTMINDEX to map to TMINDEX at mufasa");
               entityObj.setType(DK_FED_MAPPED_TEXT_ENTITY);
               
            // create fed attr
            dkAttrDef fAttr = new DKAttrDefFed();               
               fAttr.setName("FedAttr");
               fAttr.setEntityName("FedTMINDEX");
               
            // create native attr
            dkAttrDef nAttr = new DKAttrDefFed();
               nAttr.setName("TMAttr");
               nAttr.setEntityName("TMINDEX");
               
            // attribute mapping
            dkAttrMapping tmap = new DKAttrMappingFed();
               tmap.addMappedAttr(fAttr);
               tmap.addMapToAttr(nAttr);
            
            // schema mapping   
            dkSchemaMapping tsm = new DKSchemaMappingFed(ds);
               tsm.setServerName("TMMUF");
               tsm.setServerType(DK_DSTYPE_TS);
               tsm.setEntityName("TMINDEX");
               tsm.setMappedEntityName("FedTMINDEX");
               tsm.setMappedEntityType(DK_FED_MAPPED_TEXT_ENTITY);

               tsm.setAssocMappedEntityName("FedSample");
               tsm.setAssocEntityName("DLSAMPLE");
               tsm.setAssocServerName("LSMUFASA"); 
               tsm.addAttrMapping(tmap);

            // list back-end server names
            System.out.println(" *** list all back-end servers : ");
            String servers[] = adm.listServerNames();
            int len = servers.length;
            for (int i = 0; i < len; i++) {
                System.out.println("server [" + i + "] = " + servers[i]);
            }  
              
            // retrieve the definition of the servers
            DKServerMapDefFed serverdat;
            String serverName = null;
            for (int i = 0; i < len; i ++) {
                serverdat = adm.getServer(servers[i]);
                serverName = serverdat.getName().trim();
                System.out.println(" DKServerMapDefFed = " + serverName );
                System.out.println("       name : " + serverName);
                System.out.println("       type : " + serverdat.getServerType());
                System.out.println(" Connect Str: " + serverdat.getConnectString().trim());
                System.out.println("       desc : " + serverdat.getDescription());
                System.out.println("    refTime : " + serverdat.getRefreshTime());
                
            }     

            // Now, interpret the command: add, del, or list
            if (cmd.equalsIgnoreCase("add")) {
                entityObj.add();        
                entityObj.add(fAttr);
                System.out.println(" *** federated entity and attributes created ");
            }

            String[] names = ds.listEntityNames();
            
            for (int index = 0; index < names.length; index++)
               System.out.println("Entity Name[" + index + "] = " + names[index]);
               
            // list federated entity:
            System.out.println(" *** federated entities :");
            DKSequentialCollection pCol = (DKSequentialCollection) ds.listEntities();
            DKSequentialCollection pCol2;
            dkIterator pIter = pCol.createIterator();
            dkIterator pIter2 = null;
            dkEntityDef enDef = null;
            dkAttrDef attrDef = null;
            String entName;
            int i = 0, j = 0;
            
            while (pIter.more()) {
                i++;
                enDef = (dkEntityDef) pIter.next();
                entName = enDef.getName();
                System.out.println(" Entity Def name [" + i + "] - " + entName);
                System.out.println("  list attributes for " + entName + " entity");
                pCol2 = (DKSequentialCollection) ds.listEntityAttrs(entName);
                pIter2 = pCol2.createIterator();
                j = 0;
                while (pIter2.more()) {
                   j++;
                   attrDef = (dkAttrDef) pIter2.next();
                   System.out.println("  Attribute name [" + j + "] - " + attrDef.getName());
                   System.out.println("    datastoreType : " + attrDef.datastoreType());
                   System.out.println("    attributeOf   : " + attrDef.getEntityName());
                   System.out.println("    type          : " + attrDef.getType());
                   System.out.println("    size          : " + attrDef.getSize());
                   System.out.println("    id            : " + attrDef.getId());
                   System.out.println("    nullable      : " + attrDef.isNullable());
                   System.out.println("    precision     : " + attrDef.getPrecision());
                   System.out.println("    scale         : " + attrDef.getScale());
                   System.out.println("    stringType    : " + attrDef.getStringType());
                }  
                System.out.println("  " + j + " attr listed for " + entName + " entity name");
            }
            System.out.println(i + " entity def listed");

            
            try {
                if (cmd.equalsIgnoreCase("add")) {
                    tsm.add();
                    System.out.println(" ** schema mapping for TMMUF added ");
                }
                else if (cmd.equalsIgnoreCase("del")) {
                    tsm.retrieve();
                    tsm.del();
                    System.out.println(" ** schema mapping for TMMUF deleted ");
                }
            }
            catch(Exception e) { e.printStackTrace(); }


            if (cmd.equalsIgnoreCase("del")) {
                entityObj.del(fAttr);
                entityObj.del();        // deleting an entity from the federated database
                System.out.println(" *** federated entity and attributes deleted ");
            } 

            // list schema mapping names in this entity:
            System.out.println(" list schema mappings in this entity : " + entityObj.getName());

            String[] smnames = entityObj.listSchemaMappingNames();

            if (smnames != null) {
                len = smnames.length;
                for (i = 0; i < len; i++) {
                    System.out.println(" SM name : " + smnames[i]);
                }
            } else 
                System.out.println(" no sm defined in this entity ");

            // retrieve all schema mapping objects for this entity
            dkCollection coll = entityObj.listSchemaMappings();
            dkIterator iter = null;
            if (coll != null) {
                iter = coll.createIterator();
                System.out.println(" ");
                System.out.println("-- list AttrMapping -- ");
                dkAttrMapping _am;
                dkAttrDef     _ad;
                dkSchemaMapping _sm;
                while (iter.more()) {
                    _sm = (dkSchemaMapping) iter.next();
                    System.out.println("\n *** Schema Mapping Name : " + _sm.getName());
                    dkCollection amCol = _sm.listAttrMappings();
                    dkIterator amIter = amCol.createIterator();
                    
                    while (amIter.more()) {
                        _am = (dkAttrMapping) amIter.next();
                        System.out.println(" ");
                        System.out.println("  ++ AttrMapping Name : " +_am.getName());
                        System.out.println("    -- To Function : " + _am.getToNative());
                        System.out.println("    -- From Function : " + _am.getFromNative());
                        System.out.println(" -- list Mapped Attr : ");
                        int num = _am.numberOfMappedAttr();
                        System.out.println("\t Mapped attr size = " + num);
                        for (i = 0; i < num; i++) {
                            _ad = (dkAttrDef)_am.getMappedAttr((short)(i+1));
                            System.out.println("  ++ Mapped Attr Name   : " +_ad.getName());
                            System.out.println("    -- Mapped Atrr Type : " +_ad.getType());
                            System.out.println("    -- Size             : " +_ad.getSize());
                            System.out.println("    -- Precision        : " +_ad.getPrecision());
                            System.out.println("    -- Scale            : " +_ad.getScale());
                            System.out.println("    -- Max              : " +_ad.getMax());
                            System.out.println("    -- Min              : " +_ad.getMin());
                            System.out.println("    -- Nullable         : " +_ad.isNullable());
                        }
                        System.out.println(" -- list MapTo Attr -- ");
                        num = _am.numberOfMapToAttr();
                        System.out.println("\t MapTo attr size = " + num);
                        for (i = 0; i < num; i++) {
                            _ad = (dkAttrDef)_am.getMapToAttr((short)(i+1));
                            System.out.println("  ++ MapTo Attr Name   : " +_ad.getName());
                            System.out.println("    -- MapTo Atrr Type : " +_ad.getType());
                            System.out.println("    -- Size            : " +_ad.getSize());
                            System.out.println("    -- Precision       : " +_ad.getPrecision());
                            System.out.println("    -- Scale           : " +_ad.getScale());
                            System.out.println("    -- Max             : " +_ad.getMax());
                            System.out.println("    -- Min             : " +_ad.getMin());
                            System.out.println("    -- Searchable      : " +_ad.isQueryable());
                        }  
                    }
                }  
            } else System.out.println(" schema mapping is null ");


            ds.disconnect();
            System.out.println(" ** disconnects... ");
        } 
        catch (java.lang.Exception ex) 
        { 
            ex.printStackTrace(); 
        } 
    } 
}





