/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/


/*
 * Filename       : TExecPSTDemoFed.java
 * Description    : Retrieve search template from database and execute a query.
 *                : Need to run the following program first
 *                 java TADLServersFed <cmbdb> <userid> <pw> add
 *                 java TADLsmFed      <cmbdb> <userid> <pw> add
 *                 java TADLstFed      <cmbdb> <userid> <pw> add
 *
 *
 */

import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

public class TExecPSTDemoFed implements DKConstantFed 
{

   // Main method
   public static void main(String argv[]){

      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);
        
      // checking input parameters
      String libSrv = "",    // library server
             userid = "",    // userid 
             pw     = "";    // pw
        
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 3) {
         System.out.println("Usage Syntax: ");
         System.out.println("\t java TExecPSTDemoFed <cmbdb> <userid> <pw>" );
         return;
      }
      else {
         libSrv = argv[0];
         userid = argv[1];
         pw     = argv[2];
      }   

        
        
      try {
         // create federated datastore
         System.out.println(" *** connecting to datastore : " + libSrv);
         DKDatastoreFed ds = new DKDatastoreFed();
         ds.connect(libSrv, userid, pw, "");
         System.out.println(" *** datastore connected *** "); 

         // set up search criteria memory variable.
         short[] ops = { DKOpCode.EQ, DKOpCode.NEQ, DKOpCode.LIKE, DKOpCode.NOTLIKE, DKOpCode.IN, DKOpCode.NOTIN };

         DKNVPair opt[] = new DKNVPair[1];
         opt[0] = new DKNVPair("MAX_RESULTS", "6");

         //===================================================================================================
         // Retrieve search templates from federated database
         //===================================================================================================

         DKSearchTemplateFed stANDS = new DKSearchTemplateFed(ds),
                             stORS  = new DKSearchTemplateFed(ds);
         DKSearchCriteria scUserId, scTitle, scDate;
         
         // Retrieve 'FedDemoST_ANDS' search template and fill in criteria value.
         stANDS.setName("FedDemoST_ANDS");                      
         stANDS.retrieve();
         stANDS.setOptionList(opt);
         stANDS.listCriteria();                    // Retrieve all available search criteria
         
         scUserId = stANDS.retrieve("UserID");     // Retrieve 'UserID' search criteria
         scUserId.setOperator(DKOpCode.LIKE);
         scUserId.setValue("'Progr%'");            // Criteria: UserID LIKE 'Progr%'
         stANDS.update(scUserId);                  // Update criteria

         scTitle  = stANDS.retrieve("Title");      // Retrieve 'Title' search criteria
         scTitle.setOperator(DKOpCode.EQ);
         scTitle.setValue("'ROBERT'");             // Criteria: Title == 'ROBERT'
         stANDS.update(scTitle);                   // Update criteria
         
         scDate   = stANDS.retrieve("Date");       // Retrieve 'Date' search criteria
         scDate.setOperator(DKOpCode.NEQ);
         scDate.setValue("NULL");                  // Criteria: Date <> NULL
         stANDS.update(scDate);                    // Update criteria
         
         // Retrieve 'FedDemoST_ORS' search template and fill in criteria value.
         stORS.setName("FedDemoST_ORS");
         stORS.retrieve();
         stORS.setOptionList(opt);
         stORS.listCriteria();                    // Retrieve all available search criteria
         
         
         scUserId = stORS.retrieve("UserID");     // Retrieve 'UserID' search criteria
         scUserId.setOperator(DKOpCode.LIKE);
         scUserId.setValue("'Progr%'");
         stORS.update(scUserId);                   // Update criteria
         
         scTitle  = stORS.retrieve("Title");      // Retrieve 'Title' search criteria
         scTitle.setOperator(DKOpCode.EQ);
         scTitle.setValue("'BASUKI'");
         stORS.update(scTitle);                    // Update criteria
         
         scDate   = stORS.retrieve("Date");       // Retrieve 'Date' search criteria
         scDate.setOperator(DKOpCode.NEQ);
         scDate.setValue("NULL");
         stORS.update(scDate);                     // Update criteria



         //===================================================================================================
         // Execute search template.
         //===================================================================================================

         // Execute ALL ANDS search template
         DKCQExpr cqe = stANDS.createQueryExpr();

         System.out.println(" *** execute query expression for all ANDS search template ****");
         System.out.println("    Criteria: FedUserID AND FedTitle AND FedDate");
         dkResultSetCursor rsc = ds.execute(cqe);

         //  get result set cursor
         if (rsc == null) 
            System.out.println(" *** cursor is null");
         else {
	         System.out.println(" Search result hit count = " + rsc.cardinality());
            DKDDO ddo;

            while (rsc.isValid()  && !(rsc.isEnd()) )  {
               System.out.println("before fetch, position = " + rsc.getPosition() );
               ddo = rsc.fetchNext();
               System.out.println("after fetch, position = " + rsc.getPosition() );
    
               if (ddo != null) {  
                  System.out.println(" pidString : " + ddo.getPidObject().pidString());
                  displayDDO(ddo);
               } 
            }
            rsc.destroy();
         }         
         
         // Execute ALL ORS search template
         cqe = stORS.createQueryExpr();

         System.out.println(" *** execute query expression for all ORS search template ****");
         System.out.println("    Criteria: FedUserID OR FedTitle OR FedDate");
         
         rsc = ds.execute(cqe);

         //  get result set cursor
         if (rsc == null) 
            System.out.println(" *** cursor is null");
         else {
	         System.out.println(" Search result hit count = " + rsc.cardinality());
            DKDDO ddo;

            while (rsc.isValid()  && !(rsc.isEnd()) )  {
               System.out.println("before fetch, position = " + rsc.getPosition() );
               ddo = rsc.fetchNext();
               System.out.println("after fetch, position = " + rsc.getPosition() );
    
               if (ddo != null) {  
                  System.out.println(" pidString : " + ddo.getPidObject().pidString());
                  displayDDO(ddo);
               } 
            }
            rsc.destroy();
         }         

         
         ds.disconnect();
         System.out.println("Disconnect. Done.");
         ds.destroy();
      }    
      catch (DKException exc) {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc) {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }

   }

   public static void displayDDO(DKDDO ddo) throws DKException, Exception
   {
      String   dataName;
      Object a = null;
 
      if (ddo != null) {
         for (short i = 1; i <= ddo.dataCount(); i++) {
            a = ddo.getData(i);
            dataName = ddo.getDataName(i);
            System.out.print(i + ": Attribute Name :" + dataName );
            if (a != null)
               System.out.println(",\t   Value :" + a.toString());    
            else
               System.out.println(",\t   Value : null ");    
         }  
      }
   }

}

