/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TExecSTSample.java
 * Description    : Test execute query by retrieving search template from FedSample entity,
 *                  fill in criterias values and execute a search.
 *
 */

/*  How to run this testcase:
 *    1. run the server registration program:
 *             java TADLServers <feddb> <userid> <pw> add
 *    2. run either the schema mapping definition program or all of:
 *             java TADLsmSample  <feddb> <userid> <pw> add
 *             java TADLsmDemo    <feddb> <userid> <pw> add
 *             java TADLsmCombine <feddb> <userid> <pw> add
 *    3. run either the search template definition program or all of:
 *             java TADLstSample  <feddb> <userid> <pw> add
 *             java TADLstDemo    <feddb> <userid> <pw> add
 *             java TADLstCombine <feddb> <userid> <pw> add
 *    4. run this program:
 *             java TExecSTFed <feddb> <userid> <pw>
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.sql.*;
import java.util.*;

public class TExecSTFed implements DKConstantFed 
{
  // Main method
  public static void main(String argv[])
  {
    
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    // checking input parameters
    String libSrv = "",    // library server
           userid = "",    // userid 
           pw     = "";    // pw
    
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //-------------------------------------------------------------- 
    if (argv.length < 3) {
       System.out.println("Usage Syntax: ");
       System.out.println("\t java TExecSTSample <feddb> <userid> <pw> " );
       return;
    }
    else {
       libSrv = argv[0];
       userid = argv[1];
       pw     = argv[2];
    }   

    try {
        DKDatastoreFed dsFed = new DKDatastoreFed();
        System.out.println(" *** connecting to datastore :" + libSrv);

        dsFed.connect(libSrv, userid, pw, "");
        System.out.println(" --> datastore connected"); 
        String userName = dsFed.userName();
        String dsName = dsFed.datastoreName();
        System.out.println("user " + userName + " dsName " + dsName); 

        // get the Def Admin
        dkDatastoreDef  defFed = dsFed.datastoreDef();
        DKDatastoreAdminFed   adm = (DKDatastoreAdminFed) defFed.datastoreAdmin();


        dkCollection coll = dsFed.listSearchTemplates();
        DKSearchTemplateFed stObj;
        DKCQExpr cqe;
            DKNVPair opt[] = new DKNVPair[1];
            opt[0] = new DKNVPair("MAX_RESULTS", "6");
        
        boolean flag;
        Short op;
        String value;
        if (coll != null) {
            // Retrieve list of search template 
            dkIterator iter = coll.createIterator();
            while (iter.more()) {
                stObj = (DKSearchTemplateFed) iter.next();
                stObj.setOptionList(opt);
                flag = false;
                
                System.out.println("==============Retrieving Search template: " + stObj.getName());
                System.out.println("    Description: " + stObj.getDescription());
                
                dkCollection scColl = stObj.listCriteria();
                if (scColl != null) {
                    dkIterator scIter = scColl.createIterator();
                    while (scIter.more()) {
                        DKSearchCriteria sc = (DKSearchCriteria) scIter.next();
                        System.out.println("       Criteria Name: " + sc.getName());
                        op = (Short) OpTable.get(sc.getName());
                        if (op == null)
                            continue;
                    
                        value = (String) ValueTable.get(sc.getName());
                        if (value == null)
                            continue;
                        sc.setOperator(op.shortValue());
                        sc.setValue(value);
                        stObj.update(sc);
                        flag = true;
                    }
                }
                if (flag) {

                    cqe = stObj.createQueryExpr();
                    System.out.println(" SQL string of cqe = " + cqe.toSQL());
                    

                    //  execute query : this should call fed ds execute method
                
                    System.out.println(" *** execute query expression ");
                    dkResultSetCursor rsc = dsFed.execute(cqe);
                    System.out.println(" *** after execute query expression ");

                    //  get result set cursor
                    if (rsc == null) 
                        System.out.println(" *** cursor is null");
                    else {
	                    System.out.println(" hit count = " + rsc.cardinality());
                        DKDDO ddo;

                        while (rsc.isValid()  && !(rsc.isEnd()) )  {
                            System.out.println("before fetch, position = " + rsc.getPosition() );
                            ddo = rsc.fetchNext();
                            System.out.println("after fetch, position = " + rsc.getPosition() );
    
                            if (ddo != null) {  
                                System.out.println(" pidString : " + ddo.getPidObject().pidString());
                                displayDDO(ddo);
                            } else break;
                        }
                     rsc.destroy();
                    }
                }



            }
        }
        else
            System.out.println("No search template define");
        // disconnects
        System.out.println("\n\ndisconnect from the datastore Fed ...");
        dsFed.disconnect();
        System.out.println(" *** datastore Fed disconnected"); 
        dsFed.destroy();
    }
    catch (DKException exc)
    {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
    catch (Exception exc)
    {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
  }
  public static void displayDDO(DKDDO ddo) 
    throws DKException, Exception
  {
    String   dataName;
    Object a = null;
 
    if (ddo != null) {
       for (short i = 1; i <= ddo.dataCount(); i++) {
           a = ddo.getData(i);
           dataName = ddo.getDataName(i);
           System.out.print(i + ": Attribute Name :" + dataName );
           if (a != null)
              System.out.println(",\t   Value :" + a.toString());    
           else
              System.out.println(",\t   Value : null ");    
       }  
    }
  }
  
  private static Hashtable OpTable, ValueTable;
  
  static {
    Short op;
    op = new Short(DKOpCode.NEQ);
    OpTable = new Hashtable();
    ValueTable = new Hashtable();
    
    op = new Short(DKOpCode.LIKE);
    OpTable.put("Title",   (Object) op);

    op = new Short(DKOpCode.NEQ);
    OpTable.put("Subject", (Object) op);

    op = new Short(DKOpCode.LIKE);
    OpTable.put("Author",  (Object) op);

    op = new Short(DKOpCode.NEQ);
    OpTable.put("UserID",  (Object) op);

    op = new Short(DKOpCode.NEQ);
    OpTable.put("Date",  (Object) op);

    ValueTable.put("Title", "'Moving%'");
    ValueTable.put("Subject", "NULL");
    ValueTable.put("Author", "'Randall%'");
    ValueTable.put("UserID", "'BASUKI'");
    ValueTable.put("Date", "NULL");
  }

}
