/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Retrieving an Attribute Definition
    Attribute Definitions exist in two forms, the general Attribute Definition and the
    tailored Attribute Definition for a specific Component Type.  Once an Item or 
    Component Type is created using an Attribute Definition, a seperate instance will
    be stored within the Component Type definition that will not be affected by any 
    modification of the general Attribute Definition.  This prevents breaking, destroying,
    or corrupting existing Data Models and existing Items of those Data Models.
    
    As explained in both SAttributeDefinitionCreationICM & SItemTypeCreationICM, Attributes
    may be tailored for a particular situation applicable to the specific Item/Component
    Type that it belongs to.  The desired definition must be retrieved from the correct
    source.
    
    Datastore Definition's Attributes
        To obtain the general Attribute Definition that the instances added to Item Types
        are based on, retrieve the definition object from the Datastore Definition.
        
    Component Type Definition
        To obtain the specific tailored version of an Attribute Definition for a perticular
        Item or Component Type, it must be retrieved from the Component Type Definition.
        
        Hint:  Use DKComponentTypeDef.retrieveAttr().  Please refer to SItemTypeRetrievalICM
               for more information on how to retrieve an existing Component Type Definition.

    Detecting Non-Existant Attribute Definitions
        When retrieving Attribute Definitions, if it does not exist, null is returned.  
        Usage of the null object will result in a NullPointerException.  If it is unknown
        whether or not an Item or Component Type may have this Attribute the returned object
        should be checked first for null.

This Sample will retrieve general Attribute Definitions from the Datastore Definition.
    
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SAttributeDefinitionRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving & Printing Attribute Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Attribute Definitions
 *                    Printing Attribute Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SAttributeDefinitionRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attributes must already be defined.  If not, please run Sample
 *                    SAttributeDefinitionCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SAttributeDefinitionRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SAttributeDefinitionRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("=================================================");
        System.out.println("IBM Information Integrator for Content       v"+ver);
        System.out.println("Sample Program:  SAttributeDefinitionRetrievalICM");
        System.out.println("-------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("=================================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM(); // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Retrieve the S_driverLicense Attribute Definition
            //----------------------------------------------------------------
            // For more information about the defined attributes, please consult
            // the SAttributeDefinitionCreationICM sample.
            //
            // NOTE:  When retrieving Attribute Definitions, if it does not exist, null is 
            //        returned.  Usage of the null object will result in a NullPointerException.
            System.out.println("Retrieving the Attribute Definition of 'S_driverLicense'...");

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrDefICM attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr("S_driverLicense"); // Retrieve the definition.
                
            System.out.println("Retrieved the Attribute Definition of 'S_driverLicense'.");
            
            //---------------------------------------------------------------
            // Print the Attribute Definition
            //---------------------------------------------------------------
            System.out.println("Printing the Attribute Definition of 'S_driverLicense'...");
                printAttrDef(attrDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Attribute Definition of 'S_driverLicense'.");

            //----------------------------------------------------------------
            // Retrieve the S_claimNumber Attribute Definition
            //----------------------------------------------------------------
            System.out.println("Retrieving the Attribute Definition of 'S_claimNumber'...");

                attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr("S_claimNumber"); // Retrieve the definition.
                
            System.out.println("Retrieved the Attribute Definition of 'S_claimNumber'.");

            //---------------------------------------------------------------
            // Print the Attribute Definition
            //---------------------------------------------------------------
            System.out.println("Printing the Attribute Definition of 'S_claimNumber'...");
                printAttrDef(attrDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Attribute Definition of 'S_claimNumber'.");

            //----------------------------------------------------------------
            // Retrieve the S_accidentDate Attribute Definition
            //----------------------------------------------------------------
            System.out.println("Retrieving the Attribute Definition of 'S_accidentDate'...");

                attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr("S_accidentDate"); // Retrieve the definition.
                
            System.out.println("Retrieved the Attribute Definition of 'S_accidentDate'.");

            //---------------------------------------------------------------
            // Print the Attribute Definition
            //---------------------------------------------------------------
            System.out.println("Printing the Attribute Definition of 'S_accidentDate'...");
                printAttrDef(attrDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Attribute Definition of 'S_accidentDate'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print an Attribute Definition.
    * @param attrDef  Attribute Definition, DKAttrDefICM.
    **/
    public static void printAttrDef(DKAttrDefICM attrDef) throws DKException{
        System.out.println("--------------------------------------------");
        System.out.println("            Attribute Definition");
        System.out.println("---------------------------------------------");
        System.out.println("           Name:  "+attrDef.getName());
        System.out.println("    Description:  "+attrDef.getDescription());
        short type = attrDef.getType();
        switch(type){
            case DKConstant.DK_CM_BLOB:
                System.out.println("           Type:  Blob");
                break;
            case DKConstant.DK_CM_CHAR:
                System.out.println("           Type:  Char");
                break;
            case DKConstant.DK_CM_CLOB:
                System.out.println("           Type:  Clob");
                break;
            case DKConstant.DK_CM_DATE:
                System.out.println("           Type:  Date");
                break;
            case DKConstant.DK_CM_DECIMAL:
                System.out.println("           Type:  Decimal");
                break;
            case DKConstant.DK_CM_DOUBLE:
                System.out.println("           Type:  Double");
                break;
            case DKConstant.DK_CM_INTEGER:
                System.out.println("           Type:  Integer");
                break;
            case DKConstant.DK_CM_SHORT:
                System.out.println("           Type:  Short");
                break;
            case DKConstant.DK_CM_TIME:
                System.out.println("           Type:  Time");
                break;
            case DKConstant.DK_CM_TIMESTAMP:
                System.out.println("           Type:  Timestamp");
                break;
            case DKConstant.DK_CM_VARCHAR:
                System.out.println("           Type:  Varchar");
                break;
            default:
                System.out.println("           Type:  Other ("+type+")");
                break;
        }//end switch
        if(attrDef.getDefaultValue()==null)
            System.out.println("  Default Value: null");
        else{
            // Determine if Current Time, Date, Timestamp, or Zero Number are Set
            if( (attrDef.getDefaultValue().compareTo("")==0) && (attrDef.getDefaultType()!=DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_UNDEFINED)){ // If not off, assume valid
                if(type==DKConstant.DK_CM_DATE)
                    System.out.println("  Default Value:  Current Date");
                else if(type==DKConstant.DK_CM_TIME)
                    System.out.println("  Default Value:  Current Time");
                else if(type==DKConstant.DK_CM_TIMESTAMP)
                    System.out.println("  Default Value:  Current Timstamp");
                else if( (type==DKConstant.DK_CM_SHORT) || (type==DKConstant.DK_CM_INTEGER) || (type==DKConstant.DK_CM_DOUBLE) || (type==DKConstant.DK_CM_DECIMAL) )
                    System.out.println("  Default Value:  Zero Number");
                else
                    System.out.println("  Default Value:  "+attrDef.getDefaultValue());
            } else // Otherwise normal default value
                System.out.println("  Default Value:  "+attrDef.getDefaultValue());
        }
        switch(attrDef.getDefaultType()){
            case DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_UNDEFINED:
                System.out.println("           Type:  Undefined / Off");
                break;
            case DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_STRING:
                System.out.println("           Type:  String / On");
                break;
            case DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_FLOAT:
                System.out.println("           Type:  Float");
                break;
            case DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_DECIMAL:
                System.out.println("           Type:  Decimal");
                break;
            case DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_INTEGER:
                System.out.println("           Type:  Integer");
                break;
            case DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_HEXSTRING:
                System.out.println("           Type:  Hex String");
                break;
            case DKAttrDefICM.DK_ATTR_TYPE_DEFAULT_OTHER:
                System.out.println("           Type:  Other");
                break;
            default:
                System.out.println("           Type:  Unknown ("+type+")");
                break;
            
        }//end switch on Default Type
            
        if(attrDef.isNullable())
            System.out.println("       Nullable:  True");
        else
            System.out.println("       Nullable:  False");
        if(attrDef.isUnique())
            System.out.println("         Unique:  True");
        else
            System.out.println("         Unique:  False");
        if(attrDef.isTextSearchable())
            System.out.println("Text Searchable:  True");
        else
            System.out.println("Text Searchable:  False");
        if(attrDef.isReadable())
            System.out.println("       Readable:  True");
        else
            System.out.println("       Readable:  False");
        if(attrDef.isWriteable())
            System.out.println("      Writeable:  True");
        else
            System.out.println("      Writeable:  False");
        System.out.println("          Min:  "+attrDef.getMin());
        System.out.println("          Max:  "+attrDef.getMax());
        System.out.println("         Size:  "+attrDef.getSize());
        System.out.println("---------------------------------------");
    }

   /**
    * Print an Attribute Definition
    * @param dsICM     Connected DKDatastoreICM object.
    * @param attrName  Attribute Name.
    **/
    public static void printAttrDef(DKDatastoreICM dsICM, String attrName) throws DKException, Exception{
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKAttrDefICM attrDef = (DKAttrDefICM) dsDefICM.retrieveAttr(attrName); // Retrieve the definition.
        printAttrDef(attrDef);   
    }

   /**
    * Determine if the Attribute Definition exists in the Datastore Definition.
    * @param dsICM       Conncted DKDatastoreICM object.
    * @param attrName    Name of the Attribute Definition.
    * @return  Returns true if it exists, false if not.
    */
	public static boolean doesAttrDefExist(DKDatastoreICM dsICM, String attrName) throws Exception {

        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef();
        DKSequentialCollection attrDefColl = (DKSequentialCollection) dsDefICM.listAttrs();  // Get a collection containing all Attribute Definitions.

        dkIterator iter = attrDefColl.createIterator();  // Create an iterator to iterate through the collection
        while(iter.more()){                              // while there are still items in the list, continue
            dkAttrDef attrDef = (dkAttrDef) iter.next(); // Move pointer to next element & return that object.
            if(attrDef.getName().compareTo(attrName)==0)
                return(true); // found it.
        }
        
        return(false);
    }

    
}//end class SAttributeDefinitionRetrievalICM
              
