/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Retrieving an Attribute Group Definition
    Attribute Definitions exist in two forms, the general Attribute Group Definition and
    the Attribute Definition's belonging to a Group, tailored for a specific Component Type. 
    Once an Item or Component Type is created using an Attribute Group Definition, a 
    seperate instance of the Attribute Group Definition and each Attribute Definition of 
    that group will be stored within the Component Type definition that will not be affected
    by any modification of the general Attribute Group Definition or those Attribute 
    Definitions within.  This prevents breaking, destroying, or corrupting existing Data 
    Models and existing Items of those Data Models.

    Naming Convention
        The retrieved definition's list of attributes have attribute names of the following
        format, which will be needed to access and set attribute group information.

            Grouped Attribute's Name Format:  <Attribute Group Name>.<Attribute Name>

    Retrieving Attributes within a Group Directly
        Attribute Definitions within a Group may be retrieved directly without first 
        retrieving the Attribute Group Definition.  This may easily be done by simply
        retrieving a Attribute Definition directly from the Datastore Definition or 
        Component Type Definition using the Nameing Conventioned outlined above.  Please
        refer to SAttributeDefinitionRetrieval for an example of how to retrieve individual
        Attribute Definitions directly.

    Detecting Non-Existant Attribute Group Definitions
        When retrieving Attribute Group Definitions, if it does not exist, null is returned.
        Usage of the null object will result in a NullPointerException.

    Datastore Definition's Attribute Groups
        To obtain the general Attribute Group Definition that the instances added to Item
        Types are based on, retrieve the definition object from the Datastore Definition.
        
    Component Type Definition
        To obtain the specific tailored version of an Attribute Group Definition or any 
        Attribute Definition within for a perticular Item or Component Type, it must be
        retrieved from the Component Type Definition.
        
        Hint:  Use DKComponentTypeDef.retrieveAttrGroup() or retrieveAttr().  Please refer
               to SItemTypeRetrievalICM for more information on how to retrieve an existing
               Component Type Definition.

This Sample will retrieve general Attribute Group Definitions from the Datastore Definition.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SAttributeGroupDefRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving & Printing Attribute Group Definitions, Listing Attributes 
 *                    Belonging to Attribute Group, Grouped Attribute Name Format
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Attribute Group Definitions
 *                    Listing Attributes Belonging to Attribute Group
 *                    Grouped Attribute Name Format
 *                    Printing Attribute Group Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SAttributeGroupDefRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attribute Groups used here must be already defined.  If not, please run
 *                    SAttributeGroupDefCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SAttributeGroupDefRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SAttributeGroupDefCreationICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("================================================");
        System.out.println("IBM Information Integrator for Content      v"+ver);
        System.out.println("Sample Program:  SAttributeGroupDefCreationICM");
        System.out.println("------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("================================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //--------------------------------------------------------------
            // Retrieving an Attribute Group Definition
            //--------------------------------------------------------------
            // Please refer to the header documentation regarding the choice
            // between sources of the instance desired for the Definition.  
            // This example uses the Datastore Definition's general Attr.
            // Group Definition.
            //
            // Retrieving the "S_carAccident" attribute Group.
            System.out.println("Retrieving Attribute Group 'S_carAccident'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_carAccident"); // Retrieve the definition.

            System.out.println("Retrieved Attribute Group 'S_carAccident'.");

            //--------------------------------------------------------------
            // List Attributes of this Group
            //--------------------------------------------------------------
            System.out.println("Listing the Attributes Belonging to Group 'S_carAccident'...");

                DKSequentialCollection attrs = (DKSequentialCollection) attrGroupDef.listAttrs(); // Get list of attribute definitions
                
                System.out.println("\nAttributes Belonging to Group 'S_carAccident':");
                dkIterator iter = attrs.createIterator();
                while(iter.more()){  // while there are attributes, continue listing
                    DKAttrDefICM attr = (DKAttrDefICM) iter.next(); // Move pointer to next element & return that object.
                    System.out.println("     - "+ attr.getName() + ":  " + attr.getDescription());
                }
                System.out.println("");
            
            System.out.println("Listing the Attributes Belonging to Group 'S_carAccident'.");

            //--------------------------------------------------------------
            // Access & Print Attribute Group Definition Information
            //--------------------------------------------------------------
            System.out.println("Printing the Attribute Group Definition of 'S_carAccident'...");
                printAttrGroupDef(attrGroupDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Attribute Group Definition of 'S_carAccident'.");

            //--------------------------------------------------------------
            // Retrieving another Attribute Group Definition
            //--------------------------------------------------------------
            // Retrieving the "S_address" attribute Group.
            System.out.println("Retrieving Attribute Group 'S_address'...");

                attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup("S_address"); // Retrieve the definition.

            System.out.println("Retrieved Attribute Group 'S_address'.");

            //--------------------------------------------------------------
            // Access & Print Attribute Group Definition Information
            //--------------------------------------------------------------
            System.out.println("Printing the Attribute Group Definition of 'S_address'...");
                printAttrGroupDef(attrGroupDef); // Use wrapper funciton from this Sample.
            System.out.println("Printed the Attribute Group Definition of 'S_address'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print an Attribute Group Definition
    * @param attrGroupDef  Attribute Group Definition, DKAttrGroupDefICM.
    **/
    public static void printAttrGroupDef(DKAttrGroupDefICM attrGroupDef) throws DKException, Exception{
        System.out.println("---------------------------------------------");
        System.out.println("         Attribute Group Definition");
        System.out.println("---------------------------------------------");
        System.out.println("       Name:  "+attrGroupDef.getName());
        System.out.println("Description:  "+attrGroupDef.getDescription());
        System.out.println(" Attributes:   -");
        DKSequentialCollection attrs = (DKSequentialCollection) attrGroupDef.listAttrs(); // Get list of attribute definitions
        dkIterator iter = attrs.createIterator();
        while(iter.more()){  // while there are attributes, continue listing
            DKAttrDefICM attr = (DKAttrDefICM) iter.next(); // Move pointer to next element & return that object.
            System.out.println("              "+ attr.getName() + ":  " + attr.getDescription());
        }
        System.out.println("---------------------------------------------");
    }

   /**
    * Print an Attribute Group Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param attrGroupName Attribute Group Name.
    **/
    public static void printAttrGroupDef(DKDatastoreICM dsICM, String attrGroupName) throws DKException, Exception{
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKAttrGroupDefICM attrGroupDef = (DKAttrGroupDefICM) dsDefICM.retrieveAttrGroup(attrGroupName); // Retrieve the definition.
        printAttrGroupDef(attrGroupDef);   
    }

   /**
    * Obtain the Definition of the specified Attribute within the Attribute Group.  This
    * function is useful for cases such as in Item Type Creation when only one of a number
    * of Attributes within need to be accessed to set Item Type Design-Time Properties.
    * @param attrGroupDef  Attribute Group Definition object.
    * @param attrName      Name of the Attribute within the Attribute Group to obtain the Definition for.  Use name format "<Group Name>.<Attr Name>".
    * @return  Returns the Attribute Group Definition found within the Attribute group.  Null is returned if it is not found.
    **/
    public static DKAttrDefICM getAttrDefWithinGroup(DKAttrGroupDefICM attrGroupDef, String attrName) throws DKException, Exception{

        DKSequentialCollection attrs = (DKSequentialCollection) attrGroupDef.listAttrs(); // Get list of attribute definitions
                
        dkIterator iter = attrs.createIterator();
        while(iter.more()){  // while there are attributes, continue looking
            DKAttrDefICM attr = (DKAttrDefICM) iter.next(); // Move pointer to next element & return that object.
            if(attr.getName().compareTo(attrName)==0) // if the name matches the one specified, return this definition
                return(attr);
        }
        return(null);  // Otherwise it was not found, so return null.
    }
    
}//end class SAttributeGroupDefRetrievalICM
              
