/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefDeletionICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting SMS Collection Definitions, Deletes 3 Created in Creation Sample.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deletes SMS Collection Definitions.
 *                    Deletes SMS Collections Created by SSMSCollectionDefCreationICM.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SSMSCollectionDefDeletionICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: java SSMSCollectionDefSetDefaultICM MY_RM
 *                    ex: java SSMSCollectionDefSetDefaultICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined.  The 3 SMS Collections
 *                    listed as the system pre-determined names must be created, as done through
 *                    SSMSCollectionDefCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM
 *                    SResourceMgrDefRetrievalICM
 *                    SSMSCollectionDefRetrievalICM
 ************************************************************************************************/
public class SSMSCollectionDefDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argv.length < 4) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("   java SSMSCollectionDefDeletionICM");
            System.out.println("                         <Existing RM User-Defined Reference Name>");
            System.out.println("                         <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined, such as the");
            System.out.println("        default 'RMDB' specified during install.  If a new RM definition");
            System.out.println("        is to be configured, please refer to SResourceMgrDefCreationICM");
            System.out.println("        Sample.");
            System.out.println("");
            System.out.println("  NOTE: Defaults will be used if the database, username, & password are not");
            System.out.println("        specified.  All others are required.");
            System.out.println("");
            System.out.println("  ex: java SSMSCollectionDefDeletionICM MY_RM");
            System.out.println("  ex: java SSMSCollectionDefDeletionICM MY_RM icmnlsdb icmadmin password");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("=========================================================");
            System.out.println("PROGRAM TERMINATED:  RM Reference Name Must Be Specified.");
            System.out.println("=========================================================");
            return;
        } else if (argv.length != 4)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );
        
        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];

        // Store optional commandline arguments.
        if (argv.length > 1)  database = argv[1];
        if (argv.length > 2)  userName = argv[2];
        if (argv.length > 3)  password = argv[3];
        String ver = SConnectDisconnectICM.EIP_VERSION;
            
        System.out.println("=============================================");
        System.out.println("IBM Information Integrator for Content   v"+ver);
        System.out.println("Sample Program:  SSMSCollectionDefDeletionICM");
        System.out.println("---------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("=============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            System.out.println("Obtaining RM Code for RM Definition '"+rmUserDefinedReferenceName+"'...");            
            
                if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false){ // if it is not setup, stop here
                    System.out.println("NOTE: Resource Manager Definition Named '"+rmUserDefinedReferenceName+"' Not Defined.");
                    return; // since no work is to be done, simply return.
                }

                short rmCode = SResourceMgrDefRetrievalICM.getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            System.out.println("Obtained RM Code '"+rmCode+"' for RM Definition '"+rmUserDefinedReferenceName+"'.");            

            //-------------------------------------------------------------
            // Obtaining the Configuration Management Objects
            //-------------------------------------------------------------
            // The sample code blow demonstrates how to obtain objects for
            // Datastore Definition, Datastore Administration, Configuration
            // Management, and RM Configuration Management.
            System.out.println("Obtaining Configuration Management Objects...");

                DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
                DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
                DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
                DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.
                
            System.out.println("Obtained Configuration Management Objects.");

            //-------------------------------------------------------------
            // Delete the SMS Collections Definition                        
            //-------------------------------------------------------------
            System.out.println("Deleting Resource Manager Definition...");

                // The Non-Media SMS Collection is added by default when the RM Def is created.
                // It will not be deleted here.  If it is desired to deleted it, the following
                // may be used:  rmCfg.delSMSCollection(DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode);
                
                // Delete only if they are not deleted already:
                if(SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_MEDIA, rmCode))
                    rmCfg.delSMSCollection(DKConstantICM.RM_SMS_COLLECTION_MEDIA, rmCode);

                if(SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode))
                    rmCfg.delSMSCollection(DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode);

            System.out.println("Deleted Resource Manager Definition.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main


    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.
   
}//end class SSMSCollectionDefSetDefaultICM
              
