
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Description    : List the entities and attributes for an IP/390 connector.
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*; // using client package: 3-tier
import java.io.*;
import java.lang.*;

public class TClientListCatalogIP implements DKConstantIP
{

  // Main method
  public static void main(String argv[])
  {


     // Indicate this is a Console Application on Windows NT/95 for error
     // message display in dos console code page. If this was a Windows
     // GUI (Graphic User Interface) Application this would not be necessary
     // because DK_CM_SS_WINDOWS is the default.
     DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

     if (argv.length < 3)
     {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TClientListCatalogIP <libsrv> <userid> <pw>" );
        return;
     }

     String libSrv = argv[0];
     String userid = argv[1];
     String pw     = argv[2];

     DKDatastoreIP dsIP = null;
     try
     {
       System.out.println(" *** Test ImagePlus Connector ");
       dsIP = new DKDatastoreIP();

       System.out.println("---Connect to ImagePlus server---");
       String connectString = new String("APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61");
       dsIP.connect(libSrv, userid, pw, connectString);
       System.out.println("datastore ImagePlus connected");

       // List entities...
       DKEntityDefIP entDef = null;
       DKAttrDefIP attrDef = null;

       System.out.println("---List entities---");
       DKSequentialCollection pCol = (DKSequentialCollection)dsIP.listEntities();
       dkIterator pIter = null;

       if ( pCol == null )
       {
          System.out.println("collection of entities is null!");
       }
       else
       {
          // Method 1:
          System.out.println("---List attributes using listAttrNames and getAttr methods---");

          pIter = pCol.createIterator();
          while (pIter.more())
          {
             entDef = (DKEntityDefIP)pIter.next();
             System.out.println(" Entity type     : " + entDef.getType() );
             System.out.println(" Entity type name: " + entDef.getName() );

             String[] attrNames = entDef.listAttrNames();
             int count = attrNames.length;
             for (int i=0
                 ; i<count
                 ; i++ )
             {
                   attrDef = (DKAttrDefIP)entDef.getAttr( attrNames[i] );
                   System.out.println("   Attr name     : " + attrDef.getName() );
                   System.out.println("   Attr id       : " + attrDef.getId() );
                   System.out.println("   Entity name   : " + attrDef.getEntityName() );
                   System.out.println("   Datastore name: " + attrDef.datastoreName() );
                   System.out.println("   Attr type     : " + attrDef.getType() );
                   System.out.println("   Attr restrict : " + attrDef.getStringType() );
                   System.out.println("   Attr min val  : " + attrDef.getMin() );
                   System.out.println("   Attr max val  : " + attrDef.getMax() );
                   System.out.println("   Attr display  : " + attrDef.getSize() );
                   System.out.println("   Attr precision: " + attrDef.getPrecision() );
                   System.out.println("   Attr scale    : " + attrDef.getScale() );
                   System.out.println("   Attr update   ? " + attrDef.isUpdatable() );
                   System.out.println("   Attr nullable ? " + attrDef.isNullable() );
                   System.out.println("   Attr queryable? " + attrDef.isQueryable() );
                   System.out.println("" );
             }  // end for
          }  // end while

          // Method 2:
          System.out.println("---List attributes using listEntityAttrs method---");

          pIter = pCol.createIterator();
          while (pIter.more())
          {
             entDef = (DKEntityDefIP)pIter.next();
             System.out.println(" Entity type     : " + entDef.getType() );
             System.out.println(" Entity type name: " + entDef.getName() );

             DKSequentialCollection pAttrCol = (DKSequentialCollection)dsIP.listEntityAttrs(entDef.getName() );
             if ( pAttrCol == null )
             {
                System.out.println("collection of entity attrs is null for entity " + entDef.getName() );
             }
             else
             {
                dkIterator pAttrIter = pAttrCol.createIterator();
                while (pAttrIter.more())
                {
                   attrDef = (DKAttrDefIP)pAttrIter.next();
                   System.out.println("   Attr name     : " + attrDef.getName() );
                   System.out.println("   Attr id       : " + attrDef.getId() );
                   System.out.println("   Entity name   : " + attrDef.getEntityName() );
                   System.out.println("   Datastore name: " + attrDef.datastoreName() );
                   System.out.println("   Attr type     : " + attrDef.getType() );
                   System.out.println("   Attr restrict : " + attrDef.getStringType() );
                   System.out.println("   Attr min val  : " + attrDef.getMin() );
                   System.out.println("   Attr max val  : " + attrDef.getMax() );
                   System.out.println("   Attr display  : " + attrDef.getSize() );
                   System.out.println("   Attr precision: " + attrDef.getPrecision() );
                   System.out.println("   Attr scale    : " + attrDef.getScale() );
                   System.out.println("   Attr update   ? " + attrDef.isUpdatable() );
                   System.out.println("   Attr nullable ? " + attrDef.isNullable() );
                   System.out.println("   Attr queryable? " + attrDef.isQueryable() );
                   System.out.println("" );
                }  // end while
             }
          }  // end while
       }

       dsIP.disconnect();
       System.out.println("datastore ImagePlus disconnected");
       dsIP.destroy();
    }
    catch (DKException exc)
    {
     	try
       {
          if ( dsIP != null )
             dsIP.destroy();
     	}
     	catch (Exception e)
     	{
     	   e.printStackTrace();
     	}
       System.out.println("Exception name " + exc.name());
       System.out.println("Exception message " + exc.getMessage());
       exc.printStackTrace();
    }
    catch (Exception exc)
    {
     	try
       {
          if ( dsIP != null )
             dsIP.destroy();
     	}
     	catch (Exception e)
     	{
     	   e.printStackTrace();
     	}
       System.out.println("Exception message " + exc.getMessage());
       exc.printStackTrace();
    }
  }
}

