/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TDatastorePoolV4.java
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.cs.*;
import java.io.*;
import java.lang.reflect.*;

// TDatastorePoolV4 class
public class TDatastorePoolV4 implements DKConstantV4 {

  // Main method
  public static void main(String argv[]){
    
      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),   // userid 
  	     pw = new String("");       // pw
      String fullClassName = "com.ibm.mm.sdk.server.DKDatastoreV4";

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
     
      if (argv.length < 3)
      {
       System.out.println("Usage: " );
       System.out.println("  java TDatastorePoolV4 " +
        "<libSrv> <userid> <pw>" );
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
      } 
    try {
    DKDatastorePool dsrV4 = new DKDatastorePool(fullClassName);
    dsrV4.setDatastoreName(libSrv);
    dsrV4.setMaxPoolSize(10);
    dsrV4.setMinPoolSize(3);

    dsrV4.initConnections(userid, pw, 2);
    dsrV4.setTimeOut(1); 
    System.out.println(" getMaxPoolSize =  " + dsrV4.getMaxPoolSize());
    System.out.println(" getMinPoolSize =  " + dsrV4.getMinPoolSize());
    System.out.println(" getTimeOut=  " + dsrV4.getTimeOut());

    dkDatastore dsV41, dsV42, dsV43, dsV44;
    dsV41 = dsrV4.getConnection(userid, pw);             // new ds1
    System.out.println(" getConnection, dsV41(ds1) =  " + dsV41);
    dsV42 = dsrV4.getConnection(userid, pw);             // new ds2
    System.out.println(" getConnection, dsV42(ds2) =  " + dsV42);

    DKDatastoreSession dsIdV41, dsIdV42, dsIdV43, dsIdV44;
    dsIdV41 = dsrV4.getConnection(userid, pw, 0);        //new dsid1
    System.out.println(" getConnection, dsIdV41(dsid1)=" +
     dsIdV41.getDatastore() + ", sessionId=" + dsIdV41.getId());
    dsIdV42 = dsrV4.getConnection(userid, pw, 0);        //new dsid2
    System.out.println(" getConnection, dsIdV42(dsid2)=" +
     dsIdV42.getDatastore() + ", sessionId=" + dsIdV42.getId()); 

    dsrV4.returnConnection(dsIdV42);                     //dsid2 in _freeIdPool
    System.out.println(" returnConnection, dsIdV42(dsid2) =  " + 
      dsIdV42.getDatastore() + ", sessionId=" + dsIdV42.getId());
    dsrV4.returnConnection(dsV42);                       //ds2 in _freePool
    System.out.println(" returnConnection, dsV42(ds2) =  " + dsV42); 

    dsIdV43 = dsrV4.getConnection(userid, pw, 1); //ds2 in _usedPool
    System.out.println(" getConnection, dsIdV43(ds2)=" +
     dsIdV43.getDatastore() + ", sessionId=" + dsIdV43.getId());
    dsIdV44 = dsrV4.getConnection(userid, pw, 5); //new dsid3  
    System.out.println(" getConnection, dsIdV44(dsid3)=" +
     dsIdV44.getDatastore() + ", sessionId=" + dsIdV44.getId());


    dsIdV44 = dsrV4.getConnection(userid, pw, 6); // new dsid4
    System.out.println(" getConnection, dsIdV44(dsid4)=" +
     dsIdV44.getDatastore() + ", sessionId=" + dsIdV44.getId());

    dsV41 = dsrV4.getConnection(userid, pw);  // new ds3
    System.out.println(" getConnection, dsV41(ds3) =  " + dsV41);
    dsV42 = dsrV4.getConnection(userid, pw);  // new ds4
    System.out.println(" getConnection, dsV42(ds4) =  " + dsV42); 

    dsV43 = dsrV4.getConnection(userid, pw);  // new ds5
    System.out.println(" getConnection, dsV43(ds5) =  " + dsV43);
    dsV44 = dsrV4.getConnection(userid, pw);  // new ds6
    System.out.println(" getConnection, dsV44(ds6) =  " + dsV44); 


    dsIdV43 = dsrV4.getConnection(userid, pw, 9); // dsid2 
    System.out.println(" getConnection, dsIdV43(dsid2)=" +
     dsIdV43.getDatastore() + ", sessionId=" + dsIdV43.getId());


    //**** use dsV42 to do ListCatalog
/**/
    System.out.println("||**** start: use dsV42 to do ListCatalog ****||");
      DKSequentialCollection pCol = null;
      dkIterator pIter = null;
      DKSequentialCollection pCol2 = null;
      dkIterator pIter2 = null;
      DKServerDefV4 pSV = null;
      String strServerName = null;
      String strServerType = null;
      String strIndexClass = null;
      DKIndexClassDefV4 icDef = null;
      DKAttrDefV4 attrDef = null;
      int i = 0;
      int j = 0;
      System.out.println("show servers...");
      pCol = (DKSequentialCollection)dsV42.listDataSources();  // get servers list
      pIter = pCol.createIterator();
    
      while (pIter.more() == true)
      {
        i++;
        pSV = (DKServerDefV4)pIter.next();
        strServerName = pSV.getName();
        strServerType = pSV.getServerType();
        System.out.println("  Server Name[" + i + "]: " + strServerName + 
                           ", Server Type: " + strServerType);
      }
      
      System.out.println("  " + i + " server(s) listed");
      System.out.println("\nshow index classes...");
      pCol = (DKSequentialCollection) dsV42.listEntities();  // get index classes
      pIter = pCol.createIterator();
      i = 0;      
      while (pIter.more() == true)
      {
        i++;
        icDef = (DKIndexClassDefV4)pIter.next();
        strIndexClass = icDef.getName();
        System.out.println("  index class name[" + i + "]: " + strIndexClass);
        System.out.println("  show attributes for " + strIndexClass + 
                           " index class - ");
        
        // get attributes
        pCol2 = (DKSequentialCollection) dsV42.listEntityAttrs(strIndexClass);
        pIter2 = pCol2.createIterator();
        j = 0;
        
        while (pIter2.more() == true)
        {
          j++;
          attrDef = (DKAttrDefV4)pIter2.next();
          System.out.println("    Attribute name[" + j + "]: " + 
                             attrDef.getName());
          System.out.println("      datastoreType: " + attrDef.datastoreType());
          System.out.println("      attributeOf:   " + attrDef.getEntityName());
          System.out.println("      type:          " + attrDef.getType());
          System.out.println("      size:          " + attrDef.getSize());
          System.out.println("      id:            " + attrDef.getId());
          System.out.println("      nullable:      " + attrDef.isNullable());
          System.out.println("      precision:     " + attrDef.getPrecision());
          System.out.println("      scale:         " + attrDef.getScale());
          System.out.println("      stringType:    " + attrDef.getStringType());
        }
        
        System.out.println("  " + j + " attribute(s) listed for " + 
                           strIndexClass + " index class\n");
      }
      
      System.out.println(i + " index class(es) listed");
    System.out.println("||**** end: use dsV42 to do ListCatalog ****||");
/**/
    //****end of use dsV42 to do query 

    dsrV4.returnConnection(dsV42);
    System.out.println(" returnConnection, dsV42(ds4) =  " + dsV42);
    dsV44 = dsrV4.getConnection(userid, pw);
    System.out.print(" getConnection, dsV44 =  " + dsV44); 
    System.out.println("    dsV44 == dsV42 =  " + (dsV44 == dsV42)); //true

    //**** use dsV44 to retrieve XDO - need to change the itemId and partId
/**  
    System.out.println("||**** start: use dsV44 to retrieve XDO ****||");
     int    partId = 1;  
     String itemId = "V02MP2@#TOH9II$O";
     DKBlobV4 axdo = new DKBlobV4(dsV44);
     DKPidXDOV4  apid = new DKPidXDOV4();
     apid.setPartId(partId);
     apid.setItemId(itemId);
     axdo.setPidObject(apid);
     System.out.println("itemid=" + apid.getItemId());
     System.out.println("partId=" + apid.getPartId());
     System.out.println("before retrieve......");
     System.out.println("lob length=" + axdo.length());
     System.out.println("size=" + axdo.getSize());
     System.out.println("createdTimestamp=" + axdo.getCreatedTimestamp());
     System.out.println("updatedTimestamp=" + axdo.getUpdatedTimestamp());
   
     System.out.println("about to call retrieve()");
     axdo.retrieve();
     System.out.println("retrieve success...." );
     System.out.println("lob length=" + axdo.length());
     System.out.println("affiliatedTyp=" + axdo.getAffiliatedType());
    System.out.println("||**** end: use dsV44 to retrieve XDO ****||");
**/
    //****end of use dsV44 to retrieve XDO
       
    dsrV4.returnConnection(dsV41);
    System.out.println(" returnConnection, dsV41(ds3) =  " + dsV41);
    dsrV4.returnConnection(dsV43);
    System.out.println(" returnConnection, dsV43(ds5) =  " + dsV43);
    dsrV4.returnConnection(dsIdV44); //dsid4 in _freeIdPool
    System.out.println(" returnConnection, dsIdV44(dsid4) =  " + 
      dsIdV44.getDatastore() + ", sessionId=" + dsIdV44.getId());
     

    System.out.println("**** start sleep for 120050 mini sec*******");
    Thread.sleep(120050);  //the two in freePool will be disconnected
    System.out.println("**** end sleep for 120050 mini sec*******");

    dsV42 = dsrV4.getConnection(userid, pw);     //new ds7
    System.out.println(" getConnection, dsV42(ds7) =  " + dsV42); 
    System.out.println(" call destroy()..." );
    dsrV4.destroy();  //take a while for all 4 in usedPool to be disconnected

    }
    catch (DKException exc)
    {
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}

