/**
@copyright(disclaimer)
Licensed Materials - Property of IBM

IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE
IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS
BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE
DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/
import com.ibm.mm.viewer.*;
import com.ibm.mm.viewer.annotation.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;

import javax.swing.border.*;
import java.net.URL;

/**
 * Sample to demonstrate building  a stand alone viewer application with annotations support
 * using the GenericDocViewer framework.
 *
 *
 * Please read the <CMBROOT>/Samples/java/viewer/readme.html for instructions on how to setup and run
 * the samples.
 */
public class TGenericDocViewer extends JFrame
{

  private boolean traceEnabled = true;
  private CMBGenericDocViewer genericDocViewer;
  private CMBStreamingDocServices docServices;
  private CMBAnnotationServices annoServices = null;
  private String partName = new String();
  private String ext = new String();

  private static final String VIEWER_CONFIG_FILE =  "TViewerDefaultConfiguration.properties";

  public TGenericDocViewer() {
    super("EIP V8.1 Generic Document Viewer");
  }

  public void initialize()
  {
    Properties engineProperties = new Properties();
    engineProperties.put("ENGINES","4");
    engineProperties.put("ENGINE1_CLASSNAME","com.ibm.mm.viewer.CMBODDocumentEngine");
    engineProperties.put("ENGINE2_CLASSNAME","com.ibm.mm.viewer.mstech.CMBMSTechDocumentEngine");
    engineProperties.put("ENGINE3_CLASSNAME","com.ibm.mm.viewer.inso.CMBMSTechInsoEngine");
    engineProperties.put("ENGINE4_CLASSNAME","com.ibm.mm.viewer.CMBJavaDocumentEngine");

    docServices = new CMBStreamingDocServices(new StreamingDocServicesCallbacks(), engineProperties);
    annoServices = new CMBAnnotationServices(new AnnotationServicesCallbacks() );

    //Load the configuration file
    try {
     URL url = this.getClass().getClassLoader().getResource(VIEWER_CONFIG_FILE);
        InputStream configFile = null;
        Properties properties = null;
        if (url != null) {
          configFile = url.openStream();
          properties = new Properties();
          properties.load( configFile );
        }
        genericDocViewer = new CMBGenericDocViewer( docServices, annoServices, properties );
        if( configFile != null ) {
            configFile.close();
            configFile = null;
        }
    }catch( Exception e ){
       System.out.println(e);
       e.printStackTrace();
    }

    this.getContentPane().add( genericDocViewer );
    this.addWindowListener(new WindowAdapter() {
	   public void windowClosing(WindowEvent e) {
	       System.exit(0);
       }
    });
    int screenWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
    int screenHeight = Toolkit.getDefaultToolkit().getScreenSize().height;
    int viewerWidth = screenWidth*3/4;
    int viewerHeight = screenHeight*3/4;
    setLocation((screenWidth-viewerWidth)/2,0);
    pack();
    setSize(700,600);
    show();
  }


  public void openDocument()
  {
      JButton button = (JButton)((CMBViewerToolbar)genericDocViewer.getToolBar("OperationToolbar")).getComponent("open_doc");
      ActionListener openDocAction = new ActionListener(){
             public void actionPerformed(ActionEvent event) {
                    String doc = JOptionPane.showInputDialog("Enter the document file name to load:");
                    String mimetype = getMimeType( doc );
                    String nPartsStr = JOptionPane.showInputDialog("Enter the number of parts for the document:");
                    String annoFileName = JOptionPane.showInputDialog("Enter the annoFile for the document:");

                    try {
                      FileInputStream inStream = new FileInputStream(doc);
                      int extIndex = doc.lastIndexOf('.');
                      //reset ext
                      ext = "";
                      if( extIndex != -1 ){
                        partName = doc.substring(0, extIndex);
                        ext = doc.substring(extIndex,doc.length());
                      }
                      else {
                         partName = doc;
                      }
                      int nParts = 1;
                      try {
                       nParts= Integer.parseInt( nPartsStr );
                      }catch(NumberFormatException ne)
                      {
                        nParts = 1;
                      }

                      FileInputStream annoStream = null;
                      try {
                          annoStream = new FileInputStream(annoFileName );
                       }catch(Exception e )
                       {
                           annoStream = null;
                        }
                      CMBDocument document = genericDocViewer.loadDocument(inStream,nParts,mimetype,mimetype,null,null);

                      int position = document.getAnnotationPosition();

                      CMBAnnotationSet annotationSet = genericDocViewer.loadAnnotations(annoStream, "application/vnd.ibm.modcap", position, 1,0);
                      genericDocViewer.showDocument( document, annotationSet, "Document");
                      }catch(FileNotFoundException fnfe)
                      {
                        JOptionPane.showMessageDialog(genericDocViewer, "File not found:" + doc );
                      }catch(Exception e ){
                        System.out.println("File " + doc + "cannot be opened due to the following error: " + e.getMessage());
                        e.printStackTrace();
                      }
               }
      };
      button.addActionListener( openDocAction );
      genericDocViewer.registerKeyboardAction( openDocAction, KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK, false), JComponent.WHEN_IN_FOCUSED_WINDOW);
      button.setEnabled( true );
  }

   /**
     * Returns MIME content type for typical file extensions.
     */
    public static String getMimeType(String filename) {
        if( filename == null )
        return "";
        filename = filename.toLowerCase();
        if (filename.endsWith(".gif")) {
            return "image/gif";
        } else if (filename.endsWith(".jpg") || filename.endsWith(".jpeg")) {
            return "image/jpeg";
        } else if (filename.endsWith(".tif") || filename.endsWith(".tiff")) {
            return "image/tiff";
        } else if (filename.endsWith(".pdf")) {
            return "application/pdf";
        } else if (filename.endsWith(".mda")) {
            return "application/vnd.ibm.modcap";
        } else if (filename.endsWith(".afp")) {
            return "application/afp";
        } else if (filename.endsWith(".txt")) {
            return "text/plain";
        } else if (filename.endsWith(".htm") || filename.endsWith(".html")) {
            return "text/html";
        } else if (filename.endsWith(".rtf")) {
            return "text/richtext";
        } else if (filename.endsWith(".doc")) {
            return "application/msword";
        } else if (filename.endsWith(".lwp")) {
            return "application/vnd.lotus-wordpro";
        } else if (filename.endsWith(".123")) {
            return "application/vnd.lotus-1-2-3";
        } else if (filename.endsWith(".prz")) {
            return "application/vnd.lotus-freelance";
        } else if (filename.endsWith(".xls")) {
            return "application/vnd.ms-excel";
        }  else if (filename.endsWith(".wp")) {
            return "application/wordperfect5.1";
        } else if (filename.endsWith(".bmp")) {
            return "image/bmp";
        } else if (filename.endsWith(".pcx")) {
            return "image/pcx";
        } else {
            return "multipart/mixed";
        }
    }

    public static void main(String[] args) {
    TGenericDocViewer gdvFrame = new TGenericDocViewer();
    gdvFrame.initialize();
    try {
    Thread.sleep(1000);
    }catch(Exception e){}

    gdvFrame.openDocument();

  }


    /** This class handles callbacks from CMBStreamingDocServices. */
  private class StreamingDocServicesCallbacks extends CMBStreamingDocServicesCallbacks implements Serializable
  {
        StreamingDocServicesCallbacks()
        {
        }

        /** Retrieve a background form. */
        public InputStream getForm(CMBDocument document, String formName)
        {
                System.out.println("Document Engine requesting forms");
                return null;
        }

        /** Retrieve a document part. */
        public InputStream getPart(CMBDocument document, int partNumber, StringBuffer outMimeType)
        {
            System.out.println("Document Engine requesting part number:" + partName + partNumber);
            try {
            FileInputStream inStream = new FileInputStream(partName + partNumber);
             return inStream;
            }catch( IOException e)
            {
            }
           return null;
        }

        /** Retrieve a document annotation. */
        public InputStream getAnnotation(CMBDocument document, int annotationNumber)
        {
           System.out.println("Document Engine requesting annotation");
            return null;
        }

        /** Retrieve a document's resources. */
        public InputStream getResources(CMBDocument document)
        {
             System.out.println("Document Engine requesting resources");
            return null;
        }

        /** Returns true if tracing is enabled */
        public boolean traceEnabled() {
            return traceEnabled;
        }

        /** Writes a trace message. */
        public void trace(String message) {
            if (traceEnabled) {
                System.out.println("TGenericDocViewer:" + message );
            }
        }

    }
    /**
     * AnnotationServicesCallbacks class handles callbacks from CMBAnnotationServices.
     */
    private class AnnotationServicesCallbacks extends CMBAnnotationServicesCallbacks implements Serializable
    {
        AnnotationServicesCallbacks()
        {
        }

        /** Retrieves a document annotation. */
        public InputStream getAnnotationPart(CMBAnnotationSet annoSet, int annotationNumber)
                    throws CMBAnnotationEngineException
        {
            System.out.println("Annotation Engine requesting additional annotation parts");
            return null;
        }
      /** Retrieves the privilege for an annotation. */
      public boolean getPrivilege(CMBAnnotationSet annoSet, int privilegeID )
                  throws CMBAnnotationEngineException
      {
        return true;
      }

     /** Saves the annotation part. */
     public int addAnnotationPart(CMBAnnotationSet annotationSet, byte[] annotationData)
                 throws CMBAnnotationEngineException
     {
       try {
            String annoSave = JOptionPane.showInputDialog("Enter the anno file name to add:");
            FileOutputStream fout = new FileOutputStream( annoSave );
            fout.write( annotationData );
            fout.flush();
            fout.close();
        // return the ID for the new annotation
          return 0;
       } catch (Exception e) {
         e.printStackTrace();
         return -1;
       }
     }

    /** Updates a page annotation. */
     public void updateAnnotationPart(CMBAnnotationSet annotationSet, byte[] annotationData, int annotationNumber)
                 throws CMBAnnotationEngineException
     {

            String annoSave = JOptionPane.showInputDialog("Enter the anno file name to update:");
        try {
            FileOutputStream fout = new FileOutputStream( annoSave );
            fout.write( annotationData );
            fout.flush();
            fout.close();
        }catch(Exception e){
        }

     }

    /** Removes an annotation part. */
     public void removeAnnotationPart(CMBAnnotationSet annotationSet, int annotationNumber )
            throws CMBAnnotationEngineException
     {
        System.out.println("Annotation Engine attempting to delete an annotation part");

     }
     /** Returns true if tracing is enabled */
     public boolean traceEnabled() {
         return traceEnabled;
     }

     /** Writes a trace message. */
     public void trace(String message) {
         if (traceEnabled) {
             System.out.println(message );
         }
     }

    }// end of AnnotationServicesCallbacks class


}